@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarper.cpp 20188 2010-08-06 15:04:44Z rouault $
+ * $Id: gdalwarper.cpp 22888 2011-08-07 13:06:36Z rouault $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Implementation of high level convenience APIs for warper.
@@ -31,8 +31,9 @@
 #include "cpl_string.h"
 #include "cpl_minixml.h"
 #include "ogr_api.h"
+#include "gdal_priv.h"
 
-CPL_CVSID("$Id: gdalwarper.cpp 20188 2010-08-06 15:04:44Z rouault $");
+CPL_CVSID("$Id: gdalwarper.cpp 22888 2011-08-07 13:06:36Z rouault $");
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
@@ -47,8 +48,7 @@ CPL_CVSID("$Id: gdalwarper.cpp 20188 2010-08-06 15:04:44Z rouault $");
  * implement the reprojection, and will default a variety of other 
  * warp options. 
  *
- * By default all bands are transferred, with no masking or nodata values
- * in effect.  No metadata, projection info, or color tables are transferred 
+ * No metadata, projection info, or color tables are transferred
  * to the output file. 
  *
  * @param hSrcDS the source image file. 
@@ -158,7 +158,12 @@ GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
         GDALRasterBandH hBand = GDALGetRasterBand( hSrcDS, iBand+1 );
         int             bGotNoData = FALSE;
         double          dfNoDataValue;
-        
+
+        if (GDALGetRasterColorInterpretation(hBand) == GCI_AlphaBand)
+        {
+            psWOptions->nSrcAlphaBand = iBand + 1;
+        }
+
         dfNoDataValue = GDALGetRasterNoDataValue( hBand, &bGotNoData );
         if( bGotNoData )
         {
@@ -180,6 +185,12 @@ GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
 
             psWOptions->padfSrcNoDataReal[iBand] = dfNoDataValue;
         }
+
+        hBand = GDALGetRasterBand( hDstDS, iBand+1 );
+        if (hBand && GDALGetRasterColorInterpretation(hBand) == GCI_AlphaBand)
+        {
+            psWOptions->nDstAlphaBand = iBand + 1;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -409,6 +420,7 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
           float fNoData = (float) padfNoData[0];
           float *pafData = (float *) *ppImageData;
           int iOffset;
+          int bIsNoDataNan = CPLIsNan(fNoData);
 
           // nothing to do if value is out of range.
           if( padfNoData[1] != 0.0 )
@@ -416,7 +428,30 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
 
           for( iOffset = nXSize*nYSize-1; iOffset >= 0; iOffset-- )
           {
-              if( pafData[iOffset] == fNoData )
+              float fVal = pafData[iOffset];
+              if( (bIsNoDataNan && CPLIsNan(fVal)) || (!bIsNoDataNan && ARE_REAL_EQUAL(fVal, fNoData)) )
+              {
+                  panValidityMask[iOffset>>5] &= ~(0x01 << (iOffset & 0x1f));
+              }
+          }
+      }
+      break;
+      
+      case GDT_Float64:
+      {
+          double dfNoData = padfNoData[0];
+          double *padfData = (double *) *ppImageData;
+          int iOffset;
+          int bIsNoDataNan = CPLIsNan(dfNoData);
+
+          // nothing to do if value is out of range.
+          if( padfNoData[1] != 0.0 )
+              return CE_None;
+
+          for( iOffset = nXSize*nYSize-1; iOffset >= 0; iOffset-- )
+          {
+              double dfVal = padfData[iOffset];
+              if( (bIsNoDataNan && CPLIsNan(dfVal)) || (!bIsNoDataNan && ARE_REAL_EQUAL(dfVal, dfNoData)) )
               {
                   panValidityMask[iOffset>>5] &= ~(0x01 << (iOffset & 0x1f));
               }
@@ -429,6 +464,9 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
           double  *padfWrk;
           int     iLine, iPixel;
           int     nWordSize = GDALGetDataTypeSize(eType)/8;
+          
+          int bIsNoDataRealNan = CPLIsNan(padfNoData[0]);
+          int bIsNoDataImagNan = CPLIsNan(padfNoData[1]);
 
           padfWrk = (double *) CPLMalloc(nXSize * sizeof(double) * 2);
           for( iLine = 0; iLine < nYSize; iLine++ )
@@ -439,8 +477,10 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
               
               for( iPixel = 0; iPixel < nXSize; iPixel++ )
               {
-                  if( padfWrk[iPixel*2] == padfNoData[0]
-                      && padfWrk[iPixel*2+1] == padfNoData[1] )
+                  if( ((bIsNoDataRealNan && CPLIsNan(padfWrk[iPixel*2])) ||
+                       (!bIsNoDataRealNan && ARE_REAL_EQUAL(padfWrk[iPixel*2], padfNoData[0])))
+                      && ((bIsNoDataImagNan && CPLIsNan(padfWrk[iPixel*2+1])) ||
+                          (!bIsNoDataImagNan && ARE_REAL_EQUAL(padfWrk[iPixel*2+1], padfNoData[1]))) )
                   {
                       int iOffset = iPixel + iLine * nXSize;
                       
@@ -519,6 +559,95 @@ GDALWarpSrcAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
     return CE_None;
 }
 
+/************************************************************************/
+/*                       GDALWarpSrcMaskMasker()                        */
+/*                                                                      */
+/*      GDALMaskFunc for reading source simple 8bit validity mask       */
+/*      information and building a one bit validity mask.               */
+/************************************************************************/
+
+CPLErr 
+GDALWarpSrcMaskMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType, 
+                       int nXOff, int nYOff, int nXSize, int nYSize,
+                       GByte ** /*ppImageData */,
+                       int bMaskIsFloat, void *pValidityMask )
+
+{
+    GDALWarpOptions *psWO = (GDALWarpOptions *) pMaskFuncArg;
+    GUInt32  *panMask = (GUInt32 *) pValidityMask;
+
+/* -------------------------------------------------------------------- */
+/*      Do some minimal checking.                                       */
+/* -------------------------------------------------------------------- */
+    if( bMaskIsFloat )
+    {
+        CPLAssert( FALSE );
+        return CE_Failure;
+    }
+
+    if( psWO == NULL )
+    {
+        CPLAssert( FALSE );
+        return CE_Failure;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Allocate a temporary buffer to read mask byte data into.        */
+/* -------------------------------------------------------------------- */
+    GByte *pabySrcMask;
+
+    pabySrcMask = (GByte *) VSIMalloc2(nXSize,nYSize);
+    if( pabySrcMask == NULL )
+    {
+        CPLError( CE_Failure, CPLE_OutOfMemory,
+                  "Failed to allocate pabySrcMask (%dx%d) in GDALWarpSrcMaskMasker()", 
+                  nXSize, nYSize );
+        return CE_Failure;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Fetch our mask band.                                            */
+/* -------------------------------------------------------------------- */
+    GDALRasterBandH hSrcBand, hMaskBand = NULL;
+
+    hSrcBand = GDALGetRasterBand( psWO->hSrcDS, psWO->panSrcBands[0] );
+    if( hSrcBand != NULL )
+        hMaskBand = GDALGetMaskBand( hSrcBand );
+
+    if( hMaskBand == NULL )
+    {
+        CPLAssert( FALSE );
+        return CE_Failure;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Read the mask band.                                             */
+/* -------------------------------------------------------------------- */
+    CPLErr eErr;
+
+    eErr = GDALRasterIO( hMaskBand, GF_Read, nXOff, nYOff, nXSize, nYSize, 
+                         pabySrcMask, nXSize, nYSize, GDT_Byte, 0, 0 );
+
+    if( eErr != CE_None )
+    {
+        CPLFree( pabySrcMask );
+        return eErr;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Pack into 1 bit per pixel for validity.                         */
+/* -------------------------------------------------------------------- */
+    for( int iPixel = nXSize * nYSize - 1; iPixel >= 0; iPixel-- )
+    {                                    
+        if( pabySrcMask[iPixel] == 0 )
+            panMask[iPixel>>5] &= ~(0x01 << (iPixel & 0x1f));
+    }
+
+    CPLFree( pabySrcMask );
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                       GDALWarpDstAlphaMasker()                       */
 /*                                                                      */
@@ -599,7 +728,7 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
     else
     {
         for( iPixel = nXSize * nYSize - 1; iPixel >= 0; iPixel-- )
-            pafMask[iPixel] = (int) ( pafMask[iPixel] * 255.1 );
+            pafMask[iPixel] = (float)(int) ( pafMask[iPixel] * 255.1 );
         
         // Write data.
 
@@ -688,7 +817,9 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
  *
  * - CUTLINE: This may contain the WKT geometry for a cutline.  It will
  * be converted into a geometry by GDALWarpOperation::Initialize() and assigned
- * to the GDALWarpOptions hCutline field.
+ * to the GDALWarpOptions hCutline field. The coordinates must be expressed
+ * in source pixel/line coordinates. Note: this is different from the assumptions
+ * made for the -cutline option of the gdalwarp utility !
  *
  * - CUTLINE_BLEND_DIST: This may be set with a distance in pixels which
  * will be assigned to the dfCutlineBlendDist field in the GDALWarpOptions.
@@ -753,12 +884,14 @@ void CPL_STDCALL GDALDestroyWarpOptions( GDALWarpOptions *psOptions )
 
 
 #define COPY_MEM(target,type,count)					\
-   if( (psSrcOptions->target) != NULL && (count) != 0 ) 		\
+   do { if( (psSrcOptions->target) != NULL && (count) != 0 ) 		\
    { 									\
-       (psDstOptions->target) = (type *) CPLMalloc(sizeof(type)*count); \
+       (psDstOptions->target) = (type *) CPLMalloc(sizeof(type)*(count)); \
        memcpy( (psDstOptions->target), (psSrcOptions->target),		\
- 	       sizeof(type) * count ); 	        			\
-   }
+ 	       sizeof(type) * (count) ); 	        			\
+   } \
+   else \
+       (psDstOptions->target) = NULL; } while(0)
 
 /************************************************************************/
 /*                        GDALCloneWarpOptions()                        */
@@ -784,6 +917,7 @@ GDALCloneWarpOptions( const GDALWarpOptions *psSrcOptions )
     COPY_MEM( padfDstNoDataImag, double, psSrcOptions->nBandCount );
     COPY_MEM( papfnSrcPerBandValidityMaskFunc, GDALMaskFunc, 
               psSrcOptions->nBandCount );
+    psDstOptions->papSrcPerBandValidityMaskFuncArg = NULL;
 
     if( psSrcOptions->hCutline != NULL )
         psDstOptions->hCutline = 
@@ -927,24 +1061,44 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
                 CXT_Text, CPLString().Printf( "%d", psWO->panDstBands[i] ) );
         
         if( psWO->padfSrcNoDataReal != NULL )
-            CPLCreateXMLElementAndValue( 
-                psBand, "SrcNoDataReal", 
-                CPLString().Printf( "%.16g", psWO->padfSrcNoDataReal[i] ) );
+        {
+            if (CPLIsNan(psWO->padfSrcNoDataReal[i]))
+                CPLCreateXMLElementAndValue(psBand, "SrcNoDataReal", "nan");
+            else
+                CPLCreateXMLElementAndValue( 
+                    psBand, "SrcNoDataReal", 
+                    CPLString().Printf( "%.16g", psWO->padfSrcNoDataReal[i] ) );
+        }
 
         if( psWO->padfSrcNoDataImag != NULL )
-            CPLCreateXMLElementAndValue( 
-                psBand, "SrcNoDataImag", 
-                CPLString().Printf( "%.16g", psWO->padfSrcNoDataImag[i] ) );
+        {
+            if (CPLIsNan(psWO->padfSrcNoDataImag[i]))
+                CPLCreateXMLElementAndValue(psBand, "SrcNoDataImag", "nan");
+            else
+                CPLCreateXMLElementAndValue( 
+                    psBand, "SrcNoDataImag", 
+                    CPLString().Printf( "%.16g", psWO->padfSrcNoDataImag[i] ) );
+        }
 
         if( psWO->padfDstNoDataReal != NULL )
-            CPLCreateXMLElementAndValue( 
-                psBand, "DstNoDataReal", 
-                CPLString().Printf( "%.16g", psWO->padfDstNoDataReal[i] ) );
+        {
+            if (CPLIsNan(psWO->padfDstNoDataReal[i]))
+                CPLCreateXMLElementAndValue(psBand, "DstNoDataReal", "nan");
+            else
+                CPLCreateXMLElementAndValue( 
+                    psBand, "DstNoDataReal", 
+                    CPLString().Printf( "%.16g", psWO->padfDstNoDataReal[i] ) );
+        }
 
         if( psWO->padfDstNoDataImag != NULL )
-            CPLCreateXMLElementAndValue( 
-                psBand, "DstNoDataImag", 
-                CPLString().Printf( "%.16g", psWO->padfDstNoDataImag[i] ) );
+        {
+            if (CPLIsNan(psWO->padfDstNoDataImag[i]))
+                CPLCreateXMLElementAndValue(psBand, "DstNoDataImag", "nan");
+            else
+                CPLCreateXMLElementAndValue( 
+                    psBand, "DstNoDataImag", 
+                    CPLString().Printf( "%.16g", psWO->padfDstNoDataImag[i] ) );
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -1091,7 +1245,12 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 
     psWO->nBandCount = 0;
     
-    for( psBand=psBandTree->psChild; psBand != NULL; psBand = psBand->psNext )
+    if (psBandTree)
+        psBand = psBandTree->psChild;
+    else
+        psBand = NULL;
+
+    for( ; psBand != NULL; psBand = psBand->psNext )
     {
         if( psBand->eType != CXT_Element 
             || !EQUAL(psBand->pszValue,"BandMapping") )
@@ -1105,7 +1264,12 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 /* ==================================================================== */
     int iBand = 0;
 
-    for( psBand=psBandTree->psChild; psBand != NULL; psBand = psBand->psNext )
+    if (psBandTree)
+        psBand = psBandTree->psChild;
+    else
+        psBand = NULL;
+
+    for( ; psBand != NULL; psBand = psBand->psNext )
     {
         if( psBand->eType != CXT_Element 
             || !EQUAL(psBand->pszValue,"BandMapping") )
@@ -1146,7 +1310,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
                 psWO->padfSrcNoDataReal = 
                     (double *) CPLCalloc(sizeof(double),psWO->nBandCount);
 
-            psWO->padfSrcNoDataReal[iBand] = atof(pszValue);
+            psWO->padfSrcNoDataReal[iBand] = CPLAtofM(pszValue);
         }
         
         pszValue = CPLGetXMLValue(psBand,"SrcNoDataImag",NULL);
@@ -1156,7 +1320,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
                 psWO->padfSrcNoDataImag = 
                     (double *) CPLCalloc(sizeof(double),psWO->nBandCount);
 
-            psWO->padfSrcNoDataImag[iBand] = atof(pszValue);
+            psWO->padfSrcNoDataImag[iBand] = CPLAtofM(pszValue);
         }
         
 /* -------------------------------------------------------------------- */
@@ -1169,7 +1333,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
                 psWO->padfDstNoDataReal = 
                     (double *) CPLCalloc(sizeof(double),psWO->nBandCount);
 
-            psWO->padfDstNoDataReal[iBand] = atof(pszValue);
+            psWO->padfDstNoDataReal[iBand] = CPLAtofM(pszValue);
         }
         
         pszValue = CPLGetXMLValue(psBand,"DstNoDataImag",NULL);
@@ -1179,7 +1343,7 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
                 psWO->padfDstNoDataImag = 
                     (double *) CPLCalloc(sizeof(double),psWO->nBandCount);
 
-            psWO->padfDstNoDataImag[iBand] = atof(pszValue);
+            psWO->padfDstNoDataImag[iBand] = CPLAtofM(pszValue);
         }
         
         iBand++;
