@@ -1,5 +1,5 @@
 /* ****************************************************************************
- * $Id: gdal_grid.cpp 18482 2010-01-08 23:22:58Z mloskot $
+ * $Id: gdal_grid.cpp 22783 2011-07-23 19:28:16Z rouault $
  *
  * Project:  GDAL Utilities
  * Purpose:  GDAL scattered data gridding (interpolation) tool
@@ -37,18 +37,10 @@
 #include "ogr_spatialref.h"
 #include "ogr_api.h"
 #include "ogrsf_frmts.h"
+#include "gdalgrid.h"
+#include "commonutils.h"
 
-CPL_CVSID("$Id: gdal_grid.cpp 18482 2010-01-08 23:22:58Z mloskot $");
-
-static const char szAlgNameInvDist[] = "invdist";
-static const char szAlgNameAverage[] = "average";
-static const char szAlgNameNearest[] = "nearest";
-static const char szAlgNameMinimum[] = "minimum";
-static const char szAlgNameMaximum[] = "maximum";
-static const char szAlgNameRange[] = "range";
-static const char szAlgNameCount[] = "count";
-static const char szAlgNameAverageDistance[] = "average_distance";
-static const char szAlgNameAverageDistancePts[] = "average_distance_pts";
+CPL_CVSID("$Id: gdal_grid.cpp 22783 2011-07-23 19:28:16Z rouault $");
 
 /************************************************************************/
 /*                               Usage()                                */
@@ -212,176 +204,6 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
     }
 }
 
-/************************************************************************/
-/*                      ParseAlgorithmAndOptions()                      */
-/*                                                                      */
-/*      Translates mnemonic gridding algorithm names into               */
-/*      GDALGridAlgorithm code, parse control parameters and assign     */
-/*      defaults.                                                       */
-/************************************************************************/
-
-static CPLErr ParseAlgorithmAndOptions( const char *pszAlgoritm,
-                                        GDALGridAlgorithm *peAlgorithm,
-                                        void **ppOptions )
-{
-    char **papszParms = CSLTokenizeString2( pszAlgoritm, ":", FALSE );
-
-    if ( CSLCount(papszParms) < 1 )
-        return CE_Failure;
-
-    if ( EQUAL(papszParms[0], szAlgNameInvDist) )
-        *peAlgorithm = GGA_InverseDistanceToAPower;
-    else if ( EQUAL(papszParms[0], szAlgNameAverage) )
-        *peAlgorithm = GGA_MovingAverage;
-    else if ( EQUAL(papszParms[0], szAlgNameNearest) )
-        *peAlgorithm = GGA_NearestNeighbor;
-    else if ( EQUAL(papszParms[0], szAlgNameMinimum) )
-        *peAlgorithm = GGA_MetricMinimum;
-    else if ( EQUAL(papszParms[0], szAlgNameMaximum) )
-        *peAlgorithm = GGA_MetricMaximum;
-    else if ( EQUAL(papszParms[0], szAlgNameRange) )
-        *peAlgorithm = GGA_MetricRange;
-    else if ( EQUAL(papszParms[0], szAlgNameCount) )
-        *peAlgorithm = GGA_MetricCount;
-    else if ( EQUAL(papszParms[0], szAlgNameAverageDistance) )
-        *peAlgorithm = GGA_MetricAverageDistance;
-    else if ( EQUAL(papszParms[0], szAlgNameAverageDistancePts) )
-        *peAlgorithm = GGA_MetricAverageDistancePts;
-    else
-    {
-        fprintf( stderr, "Unsupported gridding method \"%s\".\n",
-                 papszParms[0] );
-        CSLDestroy( papszParms );
-        return CE_Failure;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Parse algorithm parameters and assign defaults.                 */
-/* -------------------------------------------------------------------- */
-    const char  *pszValue;
-
-    switch ( *peAlgorithm )
-    {
-        case GGA_InverseDistanceToAPower:
-        default:
-            *ppOptions =
-                CPLMalloc( sizeof(GDALGridInverseDistanceToAPowerOptions) );
-
-            pszValue = CSLFetchNameValue( papszParms, "power" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfPower = (pszValue) ? atof(pszValue) : 2.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "smoothing" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfSmoothing = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "radius1" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfRadius1 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "radius2" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfRadius2 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "angle" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfAngle = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "max_points" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                nMaxPoints = (pszValue) ? atol(pszValue) : 0;
-
-            pszValue = CSLFetchNameValue( papszParms, "min_points" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                nMinPoints = (pszValue) ? atol(pszValue) : 0;
-
-            pszValue = CSLFetchNameValue( papszParms, "nodata" );
-            ((GDALGridInverseDistanceToAPowerOptions *)*ppOptions)->
-                dfNoDataValue = (pszValue) ? atof(pszValue) : 0.0;
-            break;
-
-        case GGA_MovingAverage:
-            *ppOptions =
-                CPLMalloc( sizeof(GDALGridMovingAverageOptions) );
-
-            pszValue = CSLFetchNameValue( papszParms, "radius1" );
-            ((GDALGridMovingAverageOptions *)*ppOptions)->
-                dfRadius1 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "radius2" );
-            ((GDALGridMovingAverageOptions *)*ppOptions)->
-                dfRadius2 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "angle" );
-            ((GDALGridMovingAverageOptions *)*ppOptions)->
-                dfAngle = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "min_points" );
-            ((GDALGridMovingAverageOptions *)*ppOptions)->
-                nMinPoints = (pszValue) ? atol(pszValue) : 0;
-
-            pszValue = CSLFetchNameValue( papszParms, "nodata" );
-            ((GDALGridMovingAverageOptions *)*ppOptions)->
-                dfNoDataValue = (pszValue) ? atof(pszValue) : 0.0;
-            break;
-
-        case GGA_NearestNeighbor:
-            *ppOptions =
-                CPLMalloc( sizeof(GDALGridNearestNeighborOptions) );
-
-            pszValue = CSLFetchNameValue( papszParms, "radius1" );
-            ((GDALGridNearestNeighborOptions *)*ppOptions)->
-                dfRadius1 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "radius2" );
-            ((GDALGridNearestNeighborOptions *)*ppOptions)->
-                dfRadius2 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "angle" );
-            ((GDALGridNearestNeighborOptions *)*ppOptions)->
-                dfAngle = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "nodata" );
-            ((GDALGridNearestNeighborOptions *)*ppOptions)->
-                dfNoDataValue = (pszValue) ? atof(pszValue) : 0.0;
-            break;
-
-        case GGA_MetricMinimum:
-        case GGA_MetricMaximum:
-        case GGA_MetricRange:
-        case GGA_MetricCount:
-        case GGA_MetricAverageDistance:
-        case GGA_MetricAverageDistancePts:
-            *ppOptions =
-                CPLMalloc( sizeof(GDALGridDataMetricsOptions) );
-
-            pszValue = CSLFetchNameValue( papszParms, "radius1" );
-            ((GDALGridDataMetricsOptions *)*ppOptions)->
-                dfRadius1 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "radius2" );
-            ((GDALGridDataMetricsOptions *)*ppOptions)->
-                dfRadius2 = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "angle" );
-            ((GDALGridDataMetricsOptions *)*ppOptions)->
-                dfAngle = (pszValue) ? atof(pszValue) : 0.0;
-
-            pszValue = CSLFetchNameValue( papszParms, "min_points" );
-            ((GDALGridDataMetricsOptions *)*ppOptions)->
-                nMinPoints = (pszValue) ? atol(pszValue) : 0;
-
-            pszValue = CSLFetchNameValue( papszParms, "nodata" );
-            ((GDALGridDataMetricsOptions *)*ppOptions)->
-                dfNoDataValue = (pszValue) ? atof(pszValue) : 0.0;
-            break;
-
-   }
-
-    CSLDestroy( papszParms );
-    return CE_None;
-}
-
 /************************************************************************/
 /*                          ProcessGeometry()                           */
 /*                                                                      */
@@ -394,7 +216,7 @@ static void ProcessGeometry( OGRPoint *poGeom, OGRGeometry *poClipSrc,
                              int iBurnField, double dfBurnValue,
                              std::vector<double> &adfX,
                              std::vector<double> &adfY,
-                             std::vector<double> &adfZ)
+                             std::vector<double> &adfZ )
 
 {
     if ( poClipSrc && !poGeom->Within(poClipSrc) )
@@ -488,7 +310,7 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
         OGRFeature::DestroyFeature( poFeat );
     }
 
-    if (adfX.size() == 0)
+    if ( adfX.size() == 0 )
     {
         printf( "No point geometry found on layer %s, skipping.\n",
                 OGR_FD_GetName( OGR_L_GetLayerDefn( hSrcLayer ) ) );
@@ -498,19 +320,24 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
 /* -------------------------------------------------------------------- */
 /*      Compute grid geometry.                                          */
 /* -------------------------------------------------------------------- */
-
-    if ( !bIsXExtentSet )
+    if ( !bIsXExtentSet || !bIsYExtentSet )
     {
-        dfXMin = *std::min_element(adfX.begin(), adfX.end());
-        dfXMax = *std::max_element(adfX.begin(), adfX.end());
-        bIsXExtentSet = TRUE;
-    }
+        OGREnvelope sEnvelope;
+        OGR_L_GetExtent( hSrcLayer, &sEnvelope, TRUE );
 
-    if ( !bIsYExtentSet )
-    {
-        dfYMin = *std::min_element(adfY.begin(), adfY.end());
-        dfYMax = *std::max_element(adfY.begin(), adfY.end());
-        bIsYExtentSet = TRUE;
+        if ( !bIsXExtentSet )
+        {
+            dfXMin = sEnvelope.MinX;
+            dfXMax = sEnvelope.MaxX;
+            bIsXExtentSet = TRUE;
+        }
+
+        if ( !bIsYExtentSet )
+        {
+            dfYMin = sEnvelope.MinY;
+            dfYMax = sEnvelope.MaxY;
+            bIsYExtentSet = TRUE;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -686,6 +513,7 @@ int main( int argc, char ** argv )
 {
     GDALDriverH     hDriver;
     const char      *pszSource=NULL, *pszDest=NULL, *pszFormat = "GTiff";
+    int             bFormatExplicitelySet = FALSE;
     char            **papszLayers = NULL;
     const char      *pszBurnAttribute = NULL;
     const char      *pszWHERE = NULL, *pszSQL = NULL;
@@ -733,6 +561,7 @@ int main( int argc, char ** argv )
         else if( EQUAL(argv[i],"-of") && i < argc-1 )
         {
             pszFormat = argv[++i];
+            bFormatExplicitelySet = TRUE;
         }
 
         else if( EQUAL(argv[i],"-q") || EQUAL(argv[i],"-quiet") )
@@ -961,17 +790,37 @@ int main( int argc, char ** argv )
             Usage();
         }
     }
-    else if ( bClipSrc && poClipSrc == NULL )
+    else if ( bClipSrc && poClipSrc == NULL && !poSpatialFilter )
     {
-        if ( poSpatialFilter )
-            poClipSrc = poSpatialFilter->clone();
-        if ( poClipSrc == NULL )
+        fprintf( stderr,
+                 "FAILURE: -clipsrc must be used with -spat option or \n"
+                 "a bounding box, WKT string or datasource must be "
+                 "specified\n\n" );
+        Usage();
+    }
+
+    if ( poSpatialFilter )
+    {
+        if ( poClipSrc )
         {
-            fprintf( stderr,
-                     "FAILURE: -clipsrc must be used with -spat option or \n"
-                     "a bounding box, WKT string or datasource must be "
-                     "specified\n\n" );
-            Usage();
+            OGRGeometry *poTemp = poSpatialFilter->Intersection( poClipSrc );
+
+            if ( poTemp )
+            {
+                OGRGeometryFactory::destroyGeometry( poSpatialFilter );
+                poSpatialFilter = poTemp;
+            }
+
+            OGRGeometryFactory::destroyGeometry( poClipSrc );
+            poClipSrc = NULL;
+        }
+    }
+    else
+    {
+        if ( poClipSrc )
+        {
+            poSpatialFilter = poClipSrc;
+            poClipSrc = NULL;
         }
     }
 
@@ -1034,6 +883,9 @@ int main( int argc, char ** argv )
     if ( nYSize == 0 )
         nYSize = 256;
 
+    if (!bQuiet && !bFormatExplicitelySet)
+        CheckExtensionConsistency(pszDest, pszFormat);
+
     hDstDS = GDALCreate( hDriver, pszDest, nXSize, nYSize, nBands,
                          eOutputType, papszCreateOptions );
     if ( hDstDS == NULL )
@@ -1062,7 +914,7 @@ int main( int argc, char ** argv )
         if( hLayer != NULL )
         {
             // Custom layer will be rasterized in the first band.
-            ProcessLayer( hLayer, hDstDS, poClipSrc, nXSize, nYSize, 1,
+            ProcessLayer( hLayer, hDstDS, poSpatialFilter, nXSize, nYSize, 1,
                           bIsXExtentSet, bIsYExtentSet,
                           dfXMin, dfXMax, dfYMin, dfYMax, pszBurnAttribute,
                           eOutputType, eAlgorithm, pOptions,
@@ -1100,7 +952,7 @@ int main( int argc, char ** argv )
                 OSRExportToWkt( hSRS, &pszOutputSRS );
         }
 
-        ProcessLayer( hLayer, hDstDS, poClipSrc, nXSize, nYSize,
+        ProcessLayer( hLayer, hDstDS, poSpatialFilter, nXSize, nYSize,
                       i + 1 + nBands - nLayerCount,
                       bIsXExtentSet, bIsYExtentSet,
                       dfXMin, dfXMax, dfYMin, dfYMax, pszBurnAttribute,
@@ -1135,7 +987,6 @@ int main( int argc, char ** argv )
     OGR_DS_Destroy( hSrcDS );
     GDALClose( hDstDS );
     OGRGeometryFactory::destroyGeometry( poSpatialFilter );
-    OGRGeometryFactory::destroyGeometry( poClipSrc );
 
     CPLFree( pOptions );
     CSLDestroy( papszCreateOptions );
