@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: envisatdataset.cpp 22404 2011-05-19 15:16:06Z warmerdam $
+ * $Id: envisatdataset.cpp 22870 2011-08-06 19:07:45Z antonio $
  *
  * Project:  APP ENVISAT Support
  * Purpose:  Reader for ENVISAT format image data.
@@ -29,18 +29,126 @@
 
 #include "rawdataset.h"
 #include "cpl_string.h"
-#include "ogr_srs_api.h"					       
+#include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: envisatdataset.cpp 22404 2011-05-19 15:16:06Z warmerdam $");
+CPL_CVSID("$Id: envisatdataset.cpp 22870 2011-08-06 19:07:45Z antonio $");
 
 CPL_C_START
 #include "EnvisatFile.h"
+#include "records.h"
 CPL_C_END
 
 CPL_C_START
 void	GDALRegister_Envisat(void);
 CPL_C_END
 
+/************************************************************************/
+/* ==================================================================== */
+/*                        MerisL2FlagBand                         */
+/* ==================================================================== */
+/************************************************************************/
+class MerisL2FlagBand : public GDALPamRasterBand
+{
+  public:
+    MerisL2FlagBand( GDALDataset *, int, FILE*, off_t, off_t );
+    virtual ~MerisL2FlagBand();
+    virtual CPLErr IReadBlock( int, int, void * );
+
+  private:
+    off_t nImgOffset;
+    off_t nPrefixBytes;
+    size_t nBytePerPixel;
+    size_t nRecordSize;
+    size_t nDataSize;
+    GByte *pReadBuf;
+    FILE *fpImage;
+};
+
+/************************************************************************/
+/*                        MerisL2FlagBand()                       */
+/************************************************************************/
+MerisL2FlagBand::MerisL2FlagBand( GDALDataset *poDS, int nBand,
+                                  FILE* fpImage, off_t nImgOffset,
+                                  off_t nPrefixBytes )
+{
+    this->poDS = (GDALDataset *) poDS;
+    this->nBand = nBand;
+
+    this->fpImage = fpImage;
+    this->nImgOffset = nImgOffset;
+    this->nPrefixBytes = nPrefixBytes;
+
+    eDataType = GDT_UInt32;
+
+    nBlockXSize = poDS->GetRasterXSize();
+    nBlockYSize = 1;
+    nBytePerPixel = 3;
+
+    nDataSize = nBlockXSize * nBytePerPixel;
+    nRecordSize = nPrefixBytes + nDataSize;
+
+    pReadBuf = (GByte *) CPLMalloc( nRecordSize );
+}
+
+
+/************************************************************************/
+/*                        ~MerisL2FlagBand()                       */
+/************************************************************************/
+MerisL2FlagBand::~MerisL2FlagBand()
+{
+    CPLFree( pReadBuf );
+}
+
+/************************************************************************/
+/*                             IReadBlock()                             */
+/************************************************************************/
+CPLErr MerisL2FlagBand::IReadBlock( int nBlockXOff, int nBlockYOff,
+                                    void * pImage )
+{
+    CPLAssert( nBlockXOff == 0 );
+    CPLAssert( pReadBuf != NULL );
+
+    off_t nOffset = nImgOffset + nPrefixBytes +
+                    nBlockYOff * nBlockYSize * nRecordSize;
+
+    if ( VSIFSeek( fpImage, nOffset, SEEK_SET ) != 0 )
+    {
+        CPLError( CE_Failure, CPLE_FileIO,
+                  "Seek to %d for scanline %d failed.\n",
+                  (int)nOffset, nBlockYOff );
+        return CE_Failure;
+    }
+
+    if ( VSIFRead( pReadBuf, 1, nDataSize, fpImage ) != nDataSize )
+    {
+        CPLError( CE_Failure, CPLE_FileIO,
+                  "Read of %d bytes for scanline %d failed.\n",
+                  (int)nDataSize, nBlockYOff );
+        return CE_Failure;
+    }
+
+    unsigned iImg, iBuf;
+    for( iImg = 0, iBuf = 0;
+         iImg < nBlockXSize * sizeof(GDT_UInt32);
+         iImg += sizeof(GDT_UInt32), iBuf += nBytePerPixel )
+    {
+#ifdef CPL_LSB
+        ((GByte*) pImage)[iImg] = pReadBuf[iBuf + 2];
+        ((GByte*) pImage)[iImg + 1] = pReadBuf[iBuf + 1];
+        ((GByte*) pImage)[iImg + 2] = pReadBuf[iBuf];
+        ((GByte*) pImage)[iImg + 3] = 0;
+#else
+        ((GByte*) pImage)[iImg] = 0;
+        ((GByte*) pImage)[iImg + 1] = pReadBuf[iBuf];
+        ((GByte*) pImage)[iImg + 2] = pReadBuf[iBuf + 1];
+        ((GByte*) pImage)[iImg + 3] = pReadBuf[iBuf + 2];
+#endif
+    }
+
+    return CE_None;
+}
+
+
 /************************************************************************/
 /* ==================================================================== */
 /*				EnvisatDataset				*/
@@ -56,17 +164,18 @@ class EnvisatDataset : public RawDataset
     GDAL_GCP    *pasGCPList;
 
     char        **papszTempMD;
-    
+
     void        ScanForGCPs_ASAR();
     void        ScanForGCPs_MERIS();
 
     void	CollectMetadata( EnvisatFile_HeaderFlag );
     void        CollectDSDMetadata();
+    void        CollectADSMetadata();
 
   public:
     		EnvisatDataset();
     	        ~EnvisatDataset();
-    
+
     virtual int    GetGCPCount();
     virtual const char *GetGCPProjection();
     virtual const GDAL_GCP *GetGCPs();
@@ -164,13 +273,13 @@ void EnvisatDataset::ScanForGCPs_ASAR()
 /* -------------------------------------------------------------------- */
 /*      Do we have a meaningful geolocation grid?                       */
 /* -------------------------------------------------------------------- */
-    nDatasetIndex = EnvisatFile_GetDatasetIndex( hEnvisatFile, 
+    nDatasetIndex = EnvisatFile_GetDatasetIndex( hEnvisatFile,
                                                  "GEOLOCATION GRID ADS" );
     if( nDatasetIndex == -1 )
         return;
 
-    if( EnvisatFile_GetDatasetInfo( hEnvisatFile, nDatasetIndex, 
-                                    NULL, NULL, NULL, NULL, NULL, 
+    if( EnvisatFile_GetDatasetInfo( hEnvisatFile, nDatasetIndex,
+                                    NULL, NULL, NULL, NULL, NULL,
                                     &nNumDSR, &nDSRSize ) != SUCCESS )
         return;
 
@@ -189,7 +298,7 @@ void EnvisatDataset::ScanForGCPs_ASAR()
 
     for( iRecord = 0; iRecord < nNumDSR; iRecord++ )
     {
-        if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDatasetIndex, 
+        if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDatasetIndex,
                                            iRecord, abyRecord ) != SUCCESS )
             continue;
 
@@ -198,8 +307,8 @@ void EnvisatDataset::ScanForGCPs_ASAR()
 
         if((iRecord>1) && (int(pasGCPList[nGCPCount-1].dfGCPLine+0.5) > nRange))
         {
-            int delta = pasGCPList[nGCPCount-1].dfGCPLine -
-                        pasGCPList[nGCPCount-12].dfGCPLine;
+            int delta = (int) (pasGCPList[nGCPCount-1].dfGCPLine -
+                               pasGCPList[nGCPCount-12].dfGCPLine);
             nRange = int(pasGCPList[nGCPCount-1].dfGCPLine+0.5) + delta;
             nRangeOffset = nRange-1;
         }
@@ -211,7 +320,7 @@ void EnvisatDataset::ScanForGCPs_ASAR()
             GDALInitGCPs( 1, pasGCPList + nGCPCount );
 
             CPLFree( pasGCPList[nGCPCount].pszId );
-            
+
             sprintf( szId, "%d", nGCPCount+1 );
             pasGCPList[nGCPCount].pszId = CPLStrdup( szId );
 
@@ -228,7 +337,7 @@ void EnvisatDataset::ScanForGCPs_ASAR()
 
             pasGCPList[nGCPCount].dfGCPLine = nRange - 0.5;
             pasGCPList[nGCPCount].dfGCPPixel = nSample - 0.5;
-            
+
             nGCPCount++;
         }
     }
@@ -246,7 +355,7 @@ void EnvisatDataset::ScanForGCPs_ASAR()
         GDALInitGCPs( 1, pasGCPList + nGCPCount );
 
         CPLFree( pasGCPList[nGCPCount].pszId );
-            
+
         sprintf( szId, "%d", nGCPCount+1 );
         pasGCPList[nGCPCount].pszId = CPLStrdup( szId );
 
@@ -263,7 +372,7 @@ void EnvisatDataset::ScanForGCPs_ASAR()
 
         pasGCPList[nGCPCount].dfGCPLine = nRange - 0.5;
         pasGCPList[nGCPCount].dfGCPPixel = nSample - 0.5;
-            
+
         nGCPCount++;
     }
 }
@@ -282,13 +391,13 @@ void EnvisatDataset::ScanForGCPs_MERIS()
 /*      DS_TYPE=A and a name containing "geolocation" or "tie           */
 /*      points".                                                        */
 /* -------------------------------------------------------------------- */
-    nDatasetIndex = EnvisatFile_GetDatasetIndex( hEnvisatFile, 
+    nDatasetIndex = EnvisatFile_GetDatasetIndex( hEnvisatFile,
                                                  "Tie points ADS" );
     if( nDatasetIndex == -1 )
         return;
 
-    if( EnvisatFile_GetDatasetInfo( hEnvisatFile, nDatasetIndex, 
-                                    NULL, NULL, NULL, NULL, NULL, 
+    if( EnvisatFile_GetDatasetInfo( hEnvisatFile, nDatasetIndex,
+                                    NULL, NULL, NULL, NULL, NULL,
                                     &nNumDSR, &nDSRSize ) != SUCCESS )
         return;
 
@@ -304,23 +413,23 @@ void EnvisatDataset::ScanForGCPs_MERIS()
     if( nNumDSR == 0 )
         return;
 
-    nLinesPerTiePoint = 
-        EnvisatFile_GetKeyValueAsInt( hEnvisatFile, SPH, 
+    nLinesPerTiePoint =
+        EnvisatFile_GetKeyValueAsInt( hEnvisatFile, SPH,
                                       "LINES_PER_TIE_PT", 0 );
     nSamplesPerTiePoint =
-        EnvisatFile_GetKeyValueAsInt( hEnvisatFile, SPH, 
+        EnvisatFile_GetKeyValueAsInt( hEnvisatFile, SPH,
                                       "SAMPLES_PER_TIE_PT", 0 );
 
     if( nLinesPerTiePoint == 0 || nSamplesPerTiePoint == 0 )
         return;
 
-    nTPPerLine = (GetRasterXSize() + nSamplesPerTiePoint - 1) 
+    nTPPerLine = (GetRasterXSize() + nSamplesPerTiePoint - 1)
         / nSamplesPerTiePoint;
 
-    if( (GetRasterYSize() + nLinesPerTiePoint - 1) 
+    if( (GetRasterYSize() + nLinesPerTiePoint - 1)
         / nLinesPerTiePoint != nTPPerColumn )
     {
-        CPLDebug( "EnvisatDataset", "Got %d instead of %d nTPPerColumn.", 
+        CPLDebug( "EnvisatDataset", "Got %d instead of %d nTPPerColumn.",
                   (GetRasterYSize()+nLinesPerTiePoint-1)/nLinesPerTiePoint,
                   nTPPerColumn );
         return;
@@ -328,8 +437,8 @@ void EnvisatDataset::ScanForGCPs_MERIS()
 
     if( 50*nTPPerLine + 13 != nDSRSize )
     {
-        CPLDebug( "EnvisatDataset", 
-                  "DSRSize=%d instead of expected %d for tiepoints ADS.", 
+        CPLDebug( "EnvisatDataset",
+                  "DSRSize=%d instead of expected %d for tiepoints ADS.",
                   nDSRSize, 50*nTPPerLine + 13 );
         return;
     }
@@ -342,12 +451,12 @@ void EnvisatDataset::ScanForGCPs_MERIS()
     GUInt32 	unValue;
 
     nGCPCount = 0;
-    pasGCPList = (GDAL_GCP *) 
+    pasGCPList = (GDAL_GCP *)
         CPLCalloc(sizeof(GDAL_GCP),nNumDSR * nTPPerLine);
 
     for( iRecord = 0; iRecord < nNumDSR; iRecord++ )
     {
-        if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDatasetIndex, 
+        if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDatasetIndex,
                                            iRecord, pabyRecord ) != SUCCESS )
             continue;
 
@@ -360,23 +469,23 @@ void EnvisatDataset::ScanForGCPs_MERIS()
             GDALInitGCPs( 1, pasGCPList + nGCPCount );
 
             CPLFree( pasGCPList[nGCPCount].pszId );
-            
+
             sprintf( szId, "%d", nGCPCount+1 );
             pasGCPList[nGCPCount].pszId = CPLStrdup( szId );
 
             memcpy( &unValue, pabyRecord + 13 + nTPPerLine*4 + iGCP*4, 4 );
-            pasGCPList[nGCPCount].dfGCPX = 
+            pasGCPList[nGCPCount].dfGCPX =
                 ((int)CPL_MSBWORD32(unValue))*0.000001;
 
             memcpy( &unValue, pabyRecord + 13 + iGCP*4, 4 );
-            pasGCPList[nGCPCount].dfGCPY = 
+            pasGCPList[nGCPCount].dfGCPY =
                 ((int)CPL_MSBWORD32(unValue))*0.000001;
 
             pasGCPList[nGCPCount].dfGCPZ = 0.0;
 
             pasGCPList[nGCPCount].dfGCPLine = iRecord*nLinesPerTiePoint + 0.5;
             pasGCPList[nGCPCount].dfGCPPixel = iGCP*nSamplesPerTiePoint + 0.5;
-            
+
             nGCPCount++;
         }
     }
@@ -400,6 +509,7 @@ char **EnvisatDataset::GetMetadata( const char * pszDomain )
     int         i, nRecord = -1;
 
     strncpy( szDSName, pszDomain+11, sizeof(szDSName) );
+    szDSName[sizeof(szDSName)-1] = 0;
     for( i = 0; i < (int) sizeof(szDSName)-1; i++ )
     {
         if( szDSName[i] == '-' )
@@ -434,8 +544,8 @@ char **EnvisatDataset::GetMetadata( const char * pszDomain )
     char  *pszRecord;
 
     pszRecord = (char *) CPLMalloc(nDSRSize+1);
-    
-    if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDSIndex, nRecord, 
+
+    if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDSIndex, nRecord,
                                        pszRecord ) == FAILURE )
     {
         CPLFree( pszRecord );
@@ -450,7 +560,7 @@ char **EnvisatDataset::GetMetadata( const char * pszDomain )
 
     CSLDestroy( papszTempMD );
 
-    pszEscapedRecord = CPLEscapeString( pszRecord, nDSRSize, 
+    pszEscapedRecord = CPLEscapeString( pszRecord, nDSRSize,
                                         CPLES_BackslashQuotable );
     papszTempMD = CSLSetNameValue( NULL, "EscapedRecord", pszEscapedRecord );
     CPLFree( pszEscapedRecord );
@@ -479,24 +589,24 @@ void EnvisatDataset::CollectDSDMetadata()
     char	*pszDSName, *pszFilename;
     int		iDSD;
 
-    for( iDSD = 0; 
-         EnvisatFile_GetDatasetInfo( hEnvisatFile, iDSD, &pszDSName, NULL, 
+    for( iDSD = 0;
+         EnvisatFile_GetDatasetInfo( hEnvisatFile, iDSD, &pszDSName, NULL,
                              &pszFilename, NULL, NULL, NULL, NULL ) == SUCCESS;
          iDSD++ )
     {
-        if( pszFilename == NULL 
-            || strlen(pszFilename) == 0 
+        if( pszFilename == NULL
+            || strlen(pszFilename) == 0
             || EQUALN(pszFilename,"NOT USED",8)
             || EQUALN(pszFilename,"        ",8))
             continue;
 
         char	szKey[128], szTrimmedName[128];
         int	i;
-        
+
         strcpy( szKey, "DS_" );
         strcat( szKey, pszDSName );
 
-        // strip trailing spaces. 
+        // strip trailing spaces.
         for( i = strlen(szKey)-1; i && szKey[i] == ' '; i-- )
             szKey[i] = '\0';
 
@@ -517,6 +627,95 @@ void EnvisatDataset::CollectDSDMetadata()
     }
 }
 
+/************************************************************************/
+/*                         CollectADSMetadata()                         */
+/*                                                                      */
+/*      Collect metadata from envisat ADS and GADS.                     */
+/************************************************************************/
+
+void EnvisatDataset::CollectADSMetadata()
+{
+    int nDSIndex, nNumDsr, nDSRSize;
+    int nRecord;
+    const char *pszDSName, *pszDSType, *pszDSFilename;
+    const char *pszProduct;
+    char *pszRecord;
+    char szPrefix[128], szKey[128], szValue[1024];
+    int i;
+    CPLErr ret;
+    const EnvisatRecordDescr *pRecordDescr = NULL;
+    const EnvisatFieldDescr *pField;
+
+    pszProduct = EnvisatFile_GetKeyValueAsString( hEnvisatFile, MPH,
+                                                  "PRODUCT", "" );
+
+    for( nDSIndex = 0;
+         EnvisatFile_GetDatasetInfo( hEnvisatFile, nDSIndex,
+                                     (char **) &pszDSName,
+                                     (char **) &pszDSType,
+                                     (char **) &pszDSFilename,
+                                     NULL, NULL,
+                                     &nNumDsr, &nDSRSize ) == SUCCESS;
+         ++nDSIndex )
+    {
+        if( EQUALN(pszDSFilename,"NOT USED",8) || (nNumDsr <= 0) )
+            continue;
+        if( !EQUAL(pszDSType,"A") && !EQUAL(pszDSType,"G") )
+            continue;
+
+        for ( nRecord = 0; nRecord < nNumDsr; ++nRecord )
+        {
+            strncpy( szPrefix, pszDSName, sizeof(szPrefix) - 1);
+            szPrefix[sizeof(szPrefix) - 1] = '\0';
+
+            // strip trailing spaces
+            for( i = strlen(szPrefix)-1; i && szPrefix[i] == ' '; --i )
+                szPrefix[i] = '\0';
+
+            // convert spaces into underscores
+            for( i = 0; szPrefix[i] != '\0'; i++ )
+            {
+                if( szPrefix[i] == ' ' )
+                    szPrefix[i] = '_';
+            }
+
+            pszRecord = (char *) CPLMalloc(nDSRSize+1);
+
+            if( EnvisatFile_ReadDatasetRecord( hEnvisatFile, nDSIndex, nRecord,
+                                               pszRecord ) == FAILURE )
+            {
+                CPLFree( pszRecord );
+                return;
+            }
+
+            pRecordDescr = EnvisatFile_GetRecordDescriptor(pszProduct, pszDSName);
+            if (pRecordDescr)
+            {
+                pField = pRecordDescr->pFields;
+                while ( pField && pField->szName )
+                {
+                    ret = EnvisatFile_GetFieldAsString(pszRecord, nDSRSize,
+                                           pField, szValue);
+                    if ( ret == CE_None )
+                    {
+                        if (nNumDsr == 1)
+                            sprintf(szKey, "%s_%s", szPrefix, pField->szName);
+                        else
+                            // sprintf(szKey, "%s_%02d_%s", szPrefix, nRecord,
+                            sprintf(szKey, "%s_%d_%s", szPrefix, nRecord,
+                                    pField->szName);
+                        SetMetadataItem(szKey, szValue, "RECORDS");
+                    }
+                    // silently ignore conversion errors
+
+                    ++pField;
+                }
+            }
+            CPLFree( pszRecord );
+        }
+    }
+}
+
 /************************************************************************/
 /*                          CollectMetadata()                           */
 /*                                                                      */
@@ -527,7 +726,7 @@ void EnvisatDataset::CollectMetadata( EnvisatFile_HeaderFlag  eMPHOrSPH )
 
 {
     int		iKey;
-    
+
     for( iKey = 0; TRUE; iKey++ )
     {
         const char *pszValue, *pszKey;
@@ -568,7 +767,7 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
     EnvisatFile	*hEnvisatFile;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Check the header.                                               */
 /* -------------------------------------------------------------------- */
@@ -583,7 +782,7 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     int		ds_index;
 
-    if( EnvisatFile_Open( &hEnvisatFile, poOpenInfo->pszFilename, "r" ) 
+    if( EnvisatFile_Open( &hEnvisatFile, poOpenInfo->pszFilename, "r" )
         == FAILURE )
         return NULL;
 
@@ -596,12 +795,12 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 
     for( ds_index = 0; TRUE; ds_index++ )
     {
-        if( EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index, 
-                                        NULL, &pszDSType, NULL, 
-                                        &ds_offset, NULL, 
+        if( EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index,
+                                        NULL, &pszDSType, NULL,
+                                        &ds_offset, NULL,
                                         &num_dsr, &dsr_size ) == FAILURE )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "Unable to find \"MDS1\" measurement datatset in Envisat file." );
             EnvisatFile_Close( hEnvisatFile );
             return NULL;
@@ -611,14 +810,14 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
         if( EQUAL(pszDSType,"M") )
             break;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         EnvisatFile_Close( hEnvisatFile );
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The ENVISAT driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
@@ -639,8 +838,8 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
     GDALDataType eDataType;
     int          nPrefixBytes;
 
-    EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index, 
-                                NULL, NULL, NULL, &ds_offset, NULL, 
+    EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index,
+                                NULL, NULL, NULL, &ds_offset, NULL,
                                 &num_dsr, &dsr_size );
 
     poDS->nRasterXSize = EnvisatFile_GetKeyValueAsInt( hEnvisatFile, SPH,
@@ -650,9 +849,9 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 
     pszProduct = EnvisatFile_GetKeyValueAsString( hEnvisatFile, MPH,
                                                   "PRODUCT", "" );
-    pszDataType = EnvisatFile_GetKeyValueAsString( hEnvisatFile, SPH, 
+    pszDataType = EnvisatFile_GetKeyValueAsString( hEnvisatFile, SPH,
                                                    "DATA_TYPE", "" );
-    pszSampleType = EnvisatFile_GetKeyValueAsString( hEnvisatFile, SPH, 
+    pszSampleType = EnvisatFile_GetKeyValueAsString( hEnvisatFile, SPH,
                                                      "SAMPLE_TYPE", "" );
     if( EQUAL(pszDataType,"FLT32") && EQUALN(pszSampleType,"COMPLEX",7))
         eDataType = GDT_CFloat32;
@@ -686,13 +885,13 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
             eDataType = GDT_Byte;
     }
 
-#ifdef CPL_LSB 
+#ifdef CPL_LSB
     bNative = FALSE;
 #else
     bNative = TRUE;
 #endif
 
-    nPrefixBytes = dsr_size - 
+    nPrefixBytes = dsr_size -
         ((GDALGetDataTypeSize(eDataType) / 8) * poDS->nRasterXSize);
 
 /* -------------------------------------------------------------------- */
@@ -700,7 +899,7 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( poDS->nRasterXSize < 1 || poDS->nRasterYSize < 1 )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Unable to determine organization of dataset.  It would\n"
                   "appear this is an Envisat dataset, but an unsupported\n"
                   "data product.  Unable to utilize." );
@@ -721,37 +920,129 @@ GDALDataset *EnvisatDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Scan for all datasets matching the reference dataset.           */
 /* -------------------------------------------------------------------- */
-    int	num_dsr2, dsr_size2, iBand = 0;
+    int num_dsr2, dsr_size2, iBand = 0;
     const char *pszDSName;
+    char szBandName[128];
+    bool bMiltiChannel;
 
-    for( ds_index = 0; 
-         EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index, 
-                                     (char **) &pszDSName, NULL, NULL, 
-                                     &ds_offset, NULL, 
+    for( ds_index = 0;
+         EnvisatFile_GetDatasetInfo( hEnvisatFile, ds_index,
+                                     (char **) &pszDSName, NULL, NULL,
+                                     &ds_offset, NULL,
                                      &num_dsr2, &dsr_size2 ) == SUCCESS;
          ds_index++ )
     {
-        if( EQUAL(pszDSType,"M") 
-            && num_dsr2 == num_dsr && dsr_size2 == dsr_size )
+        if( !EQUAL(pszDSType,"M") || num_dsr2 != num_dsr )
+            continue;
+
+        if( EQUALN(pszProduct,"MER",3) && (pszProduct[8] == '2') &&
+            ( (strstr(pszDSName, "MDS(16)") != NULL) ||
+              (strstr(pszDSName, "MDS(19)") != NULL)) )
+            bMiltiChannel = true;
+        else
+            bMiltiChannel = false;
+
+        if( (dsr_size2 == dsr_size) && !bMiltiChannel )
         {
             poDS->SetBand( iBand+1,
                        new RawRasterBand( poDS, iBand+1, poDS->fpImage,
                                           ds_offset + nPrefixBytes,
-                                          GDALGetDataTypeSize(eDataType) / 8, 
-                                          dsr_size, 
+                                          GDALGetDataTypeSize(eDataType) / 8,
+                                          dsr_size,
                                           eDataType, bNative ) );
             iBand++;
 
             poDS->GetRasterBand(iBand)->SetDescription( pszDSName );
         }
+/* -------------------------------------------------------------------- */
+/*       Handle MERIS Level 2 datasets with data type different from    */
+/*       the one declared in the SPH                                    */
+/* -------------------------------------------------------------------- */
+        else if( EQUALN(pszProduct,"MER",3) &&
+                 (strstr(pszDSName, "Flags") != NULL) )
+        {
+            if (pszProduct[8] == '1')
+            {
+                // Flags
+                poDS->SetBand( iBand+1,
+                           new RawRasterBand( poDS, iBand+1, poDS->fpImage,
+                                              ds_offset + nPrefixBytes, 3,
+                                              dsr_size, GDT_Byte, bNative ) );
+                iBand++;
+
+                poDS->GetRasterBand(iBand)->SetDescription( pszDSName );
+
+                // Detector indices
+                poDS->SetBand( iBand+1,
+                           new RawRasterBand( poDS, iBand+1, poDS->fpImage,
+                                              ds_offset + nPrefixBytes + 1,
+                                              3, dsr_size, GDT_Int16,
+                                              bNative ) );
+                iBand++;
+
+                const char *pszSuffix = strstr( pszDSName, "MDS" );
+                if ( pszSuffix != NULL)
+                    sprintf( szBandName, "Detector index %s", pszSuffix );
+                else
+                    sprintf( szBandName, "Detector index" );
+                poDS->GetRasterBand(iBand)->SetDescription( szBandName );
+            }
+            else if ( (pszProduct[8] == '2') &&
+                      (dsr_size2 >= 3 * poDS->nRasterXSize ) )
+            {
+                int nFlagPrefixBytes = dsr_size2 - 3 * poDS->nRasterXSize;
+
+                poDS->SetBand( iBand+1,
+                       new MerisL2FlagBand( poDS, iBand+1, poDS->fpImage,
+                                            ds_offset, nFlagPrefixBytes ) );
+                iBand++;
+
+                poDS->GetRasterBand(iBand)->SetDescription( pszDSName );
+            }
+        }
+        else if( EQUALN(pszProduct,"MER",3) && (pszProduct[8] == '2') )
+        {
+            int nPrefixBytes2, nSubBands, nSubBandIdx, nSubBandOffset;
+
+            int nPixelSize = 1;
+            GDALDataType eDataType2 = GDT_Byte;
+
+            nSubBands = dsr_size2 / poDS->nRasterXSize;
+            if( (nSubBands < 1) || (nSubBands > 3) )
+                nSubBands = 0;
+
+            nPrefixBytes2 = dsr_size2 -
+                (nSubBands * nPixelSize * poDS->nRasterXSize);
+
+            for (nSubBandIdx = 0; nSubBandIdx < nSubBands; ++nSubBandIdx)
+            {
+                nSubBandOffset =
+                    ds_offset + nPrefixBytes2 + nSubBandIdx * nPixelSize;
+                poDS->SetBand( iBand+1,
+                        new RawRasterBand( poDS, iBand+1, poDS->fpImage,
+                                           nSubBandOffset,
+                                           nPixelSize * nSubBands,
+                                           dsr_size2, eDataType2, bNative ) );
+                iBand++;
+
+                if (nSubBands > 1)
+                {
+                    sprintf( szBandName, "%s (%d)", pszDSName, nSubBandIdx );
+                    poDS->GetRasterBand(iBand)->SetDescription( szBandName );
+                }
+                else
+                    poDS->GetRasterBand(iBand)->SetDescription( pszDSName );
+            }
+        }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Collect metadata.                                               */
 /* -------------------------------------------------------------------- */
     poDS->CollectMetadata( MPH );
     poDS->CollectMetadata( SPH );
     poDS->CollectDSDMetadata();
+    poDS->CollectADSMetadata();
 
     if( EQUALN(pszProduct,"MER",3) )
         poDS->ScanForGCPs_MERIS();
@@ -784,11 +1075,11 @@ void GDALRegister_Envisat()
     if( GDALGetDriverByName( "ESAT" ) == NULL )
     {
         poDriver = new GDALDriver();
-        
+
         poDriver->SetDescription( "ESAT" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
+        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                    "Envisat Image Format" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
+        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
                                    "frmt_various.html#Envisat" );
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "n1" );
 
