@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalallregister.cpp 18207 2009-12-07 21:37:49Z rouault $
+ * $Id: gdalallregister.cpp 23473 2011-12-05 22:17:02Z rouault $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALAllRegister(), primary format registration.
@@ -30,7 +30,7 @@
 #include "gdal_priv.h"
 #include "gdal_frmts.h"
 
-CPL_CVSID("$Id: gdalallregister.cpp 18207 2009-12-07 21:37:49Z rouault $");
+CPL_CVSID("$Id: gdalallregister.cpp 23473 2011-12-05 22:17:02Z rouault $");
 
 #ifdef notdef
 // we may have a use for this some day
@@ -90,6 +90,7 @@ void CPL_STDCALL GDALAllRegister()
 #ifdef FRMT_nitf
     GDALRegister_NITF();
     GDALRegister_RPFTOC();
+    GDALRegister_ECRGTOC();
 #endif
 
 #ifdef FRMT_hfa
@@ -123,6 +124,7 @@ void CPL_STDCALL GDALAllRegister()
 
 #ifdef FRMT_aaigrid
     GDALRegister_AAIGrid();
+    GDALRegister_GRASSASCIIGrid();
 #endif
 
 #ifdef FRMT_sdts
@@ -141,6 +143,10 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_PNG();
 #endif
 
+#ifdef FRMT_gta
+    GDALRegister_GTA();
+#endif
+
 #ifdef FRMT_jpeg
     GDALRegister_JPEG();
 #endif
@@ -153,6 +159,10 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_JDEM();
 #endif
 
+#ifdef FRMT_rasdaman
+    GDALRegister_RASDAMAN();
+#endif
+
 #ifdef FRMT_gif
     GDALRegister_GIF();
     GDALRegister_BIGGIF();
@@ -247,11 +257,21 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_JP2KAK();
 #endif
 
+#ifdef FRMT_jpipkak
+// JPEG2000 support using Kakadu toolkit
+    GDALRegister_JPIPKAK();
+#endif
+
 #ifdef FRMT_ecw
     GDALRegister_ECW();
     GDALRegister_JP2ECW();
 #endif
 
+#ifdef FRMT_openjpeg
+// JPEG2000 support using OpenJPEG library
+    GDALRegister_JP2OpenJPEG();
+#endif
+
 #ifdef FRMT_jpeg2000
 // JPEG2000 support using JasPer toolkit
 // This one should always be placed after other JasPer supported formats,
@@ -275,6 +295,10 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_MrSID();
 #endif
 
+#ifdef FRMT_mrsid_lidar
+    GDALRegister_MG4Lidar();
+#endif
+
 #ifdef FRMT_rmf
     GDALRegister_RMF();
 #endif
@@ -361,6 +385,11 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_EIR();
     GDALRegister_DIPEx();
     GDALRegister_LCP();
+    GDALRegister_GTX();
+    GDALRegister_LOSLAS();
+    GDALRegister_NTv2();
+    GDALRegister_ACE2();
+    GDALRegister_SNODAS();
 #endif
 
 /* -------------------------------------------------------------------- */
@@ -428,13 +457,58 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_EPSILON();
 #endif
 
-#ifdef FRMT_wktraster
-    GDALRegister_WKTRaster();
+#ifdef FRMT_postgisraster
+    GDALRegister_PostGISRaster();
 #endif
 
 #ifdef FRMT_saga
     GDALRegister_SAGA();
 #endif
+
+#ifdef FRMT_kmlsuperoverlay
+    GDALRegister_KMLSUPEROVERLAY();
+#endif
+
+#ifdef FRMT_xyz
+    GDALRegister_XYZ();
+#endif
+
+#ifdef FRMT_hf2
+    GDALRegister_HF2();
+#endif
+
+#ifdef FRMT_pdf
+    GDALRegister_PDF();
+#endif
+
+#ifdef FRMT_jpegls
+    GDALRegister_JPEGLS();
+#endif
+
+#ifdef FRMT_ozi
+    GDALRegister_OZI();
+#endif
+
+#ifdef FRMT_ctg
+    GDALRegister_CTG();
+#endif
+
+#ifdef FRMT_e00grid
+    GDALRegister_E00GRID();
+#endif
+
+#ifdef FRMT_webp
+    GDALRegister_WEBP();
+#endif
+
+#ifdef FRMT_zmap
+    GDALRegister_ZMap();
+#endif
+
+#ifdef FRMT_ngsgeoid
+    GDALRegister_NGSGEOID();
+#endif
+
 /* -------------------------------------------------------------------- */
 /*      Deregister any drivers explicitly marked as supressed by the    */
 /*      GDAL_SKIP environment variable.                                 */
