@@ -35,7 +35,6 @@
 
 #include "gdal.h"
 #include "gdal_priv.h"
-#include "cpl_string.h"
 
 // Oracle Class Interface
 
@@ -86,7 +85,11 @@ const char*         OWReplaceString( const char* pszBaseString,
 #define TYPE_OWNER                  "MDSYS"
 #define SDO_GEOMETRY                TYPE_OWNER".SDO_GEOMETRY"
 #define SDO_GEORASTER               TYPE_OWNER".SDO_GEORASTER"
+#define SDO_PC                      TYPE_OWNER".SDO_PC"
 #define SDO_NUMBER_ARRAY            TYPE_OWNER".SDO_NUMBER_ARRAY"
+#define SDO_ORDINATE_ARRAY          TYPE_OWNER".SDO_ORDINATE_ARRAY"
+#define SDO_ELEM_INFO_ARRAY         TYPE_OWNER".SDO_ELEM_INFO_ARRAY"
+
 #define OW_XMLNS        "xmlns=\"http://xmlns.oracle.com/spatial/georaster\""
 
 /***************************************************************************/
@@ -97,6 +100,14 @@ typedef OCIRef SDO_GEORASTER_ref;
 typedef OCIRef SDO_GEOMETRY_ref;
 typedef OCIRef SDO_POINT_TYPE_ref;
 
+typedef OCIArray sdo_elem_info_array;
+typedef OCIArray sdo_ordinate_array;
+typedef OCIArray SDO_NUMBER_ARRAY_TYPE;
+
+/***************************************************************************/
+/*                            Point type                                   */
+/***************************************************************************/
+
 struct sdo_point_type
 {
     OCINumber x;
@@ -106,9 +117,19 @@ struct sdo_point_type
 
 typedef struct sdo_point_type sdo_point_type;
 
-typedef OCIArray sdo_elem_info_array;
-typedef OCIArray sdo_ordinate_array;
-typedef OCIArray SDO_NUMBER_ARRAY_TYPE;
+struct sdo_point_type_ind
+{
+    OCIInd      _atomic;
+    OCIInd      x;
+    OCIInd      y;
+    OCIInd      z;
+};
+
+typedef struct sdo_point_type_ind sdo_point_type_ind;
+
+/***************************************************************************/
+/*                            Geometry type                                */
+/***************************************************************************/
 
 struct sdo_geometry
 {
@@ -118,16 +139,8 @@ struct sdo_geometry
     OCIArray*       sdo_elem_info;
     OCIArray*       sdo_ordinates;
 };
-typedef struct sdo_geometry SDO_GEOMETRY_TYPE;
 
-struct sdo_point_type_ind
-{
-    OCIInd      _atomic;
-    OCIInd      x;
-    OCIInd      y;
-    OCIInd      z;
-};
-typedef struct sdo_point_type_ind sdo_point_type_ind;
+typedef struct sdo_geometry SDO_GEOMETRY_TYPE;
 
 struct sdo_geometry_ind
 {
@@ -138,8 +151,13 @@ struct sdo_geometry_ind
     OCIInd      sdo_elem_info;
     OCIInd      sdo_ordinates;
 };
+
 typedef struct SDO_GEOMETRY_ind SDO_GEOMETRY_ind;
 
+/***************************************************************************/
+/*                            GeoRaster type                               */
+/***************************************************************************/
+
 struct sdo_georaster
 {
     OCINumber          rastertype;
@@ -148,6 +166,7 @@ struct sdo_georaster
     OCINumber          rasterid;
     void*              metadata;
 };
+
 typedef struct sdo_georaster SDO_GEORASTER_TYPE;
 
 struct sdo_georaster_ind
@@ -159,8 +178,78 @@ struct sdo_georaster_ind
     OCIInd            rasterid;
     OCIInd            metadata;
 };
+
 typedef struct sdo_georaster_ind SDO_GEORASTER_ind;
 
+/***************************************************************************/
+/*                            Point Cloud type                             */
+/***************************************************************************/
+
+struct sdo_mbr
+{
+   OCIArray*            lower_left;
+   OCIArray*            upper_right;
+};
+typedef struct sdo_mbr SDO_MBR_TYPE;
+
+struct sdo_mbr_ind
+{
+   OCIInd               _atomic;
+   OCIInd               lower_left;
+   OCIInd               upper_right;
+};
+typedef struct sdo_mbr_ind SDO_MBR_ind;
+
+struct sdo_orgscl_type
+{
+   SDO_MBR_TYPE         extent;
+   OCIArray*            scale;
+   OCIArray*            ord_cmp_type;
+};
+typedef struct sdo_orgscl_type SDO_ORGSCL_TYPE;
+
+struct sdo_orgscl_type_ind
+{
+   OCIInd               _atomic;
+   SDO_MBR_ind          extent;
+   OCIInd               scale;
+   OCIInd               ord_cmp_type;
+};
+typedef struct sdo_orgscl_type_ind SDO_ORGSCL_TYPE_ind;
+
+struct sdo_pc
+{
+    OCIString*          base_table;
+    OCIString*          base_column;
+    OCINumber           pc_id;
+    OCIString*          blk_table;
+    OCIString*          ptn_params;
+    SDO_GEOMETRY_TYPE   pc_geometry;
+    OCINumber           pc_tol;
+    OCINumber           pc_tot_dimensions;
+    SDO_ORGSCL_TYPE     pc_domain;
+    OCIString*          pc_val_attr_tables;
+    void*               pc_other_attrs;
+};
+typedef struct sdo_pc SDO_PC_TYPE;
+
+struct sdo_pc_ind
+{
+    OCIInd              _atomic;
+    OCIInd              base_table;
+    OCIInd              base_column;
+    OCIInd              pc_id;
+    OCIInd              blk_table;
+    OCIInd              ptn_params;
+    sdo_geometry_ind    pc_geometry;
+    OCIInd              pc_tol;
+    OCIInd              pc_tot_dimensions;
+    OCIInd              pc_domain;
+    OCIInd              pc_val_attr_tables;
+    OCIInd              pc_other_attrs;
+};
+typedef struct sdo_pc_ind SDO_PC_ind;
+
 /***************************************************************************/
 /*                            Oracle class wrappers                        */
 /***************************************************************************/
@@ -189,6 +278,8 @@ private:
     OCIEnv*             hEnv;
     OCIError*           hError;
     OCISvcCtx*          hSvcCtx;
+    OCIServer*          hServer;
+    OCISession*         hSession;
     OCIDescribe*        hDescribe;
 
     int                 nVersion;
@@ -203,10 +294,13 @@ private:
     OCIType*            hNumArrayTDO;
     OCIType*            hGeometryTDO;
     OCIType*            hGeoRasterTDO;
+    OCIType*            hPCTDO;
+    OCIType*            hElemArrayTDO;
+    OCIType*            hOrdnArrayTDO;
 
 public:
 
-    OWStatement*        CreateStatement( const char* pszStatementIn );
+    OWStatement*        CreateStatement( const char* pszStatement );
     OCIParam*           GetDescription( char* pszTable );
     bool                GetNextField(
                             OCIParam* phTable,
@@ -219,7 +313,9 @@ public:
 
     void                CreateType( sdo_geometry** pphData );
     void                DestroyType( sdo_geometry** pphData );
-    OCIType*            DescribeType( char *pszTypeName );
+    void                CreateType( OCIArray** phData , OCIType* type);
+    void                DestroyType( OCIArray** phData );
+    OCIType*            DescribeType( const char *pszTypeName );
 
     bool                Succeeded() { return bSuceeeded; };
 
@@ -229,6 +325,15 @@ public:
     int                 GetVersion () { return nVersion; };
     sb4                 GetCharSize () { return nCharSize; };
 
+    OCIType*            GetGeometryType() { return hGeometryTDO; }
+    OCIType*            GetGeoRasterType() { return hGeoRasterTDO; }
+    OCIType*            GetElemInfoType() {return hElemArrayTDO; }
+    OCIType*            GetOrdinateType() {return hOrdnArrayTDO; }
+
+    bool                Commit(); // OCITransCommit()
+    bool                StartTransaction(); //  //OCITransStart()
+    bool                EndTransaction() {return Commit(); }
+
 };
 
 /***************************************************************************/
@@ -240,13 +345,13 @@ class OWStatement
 
 public:
 
-                        OWStatement( OWConnection* pConnection,
-                            const char* pszStatementIn );
+                        OWStatement( OWConnection* poConnect, 
+                            const char* pszStatement );
     virtual            ~OWStatement();
 
 private:
 
-    OWConnection*       poConnect;
+    OWConnection*       poConnection;
     OCIStmt*            hStmt;
     OCIError*           hError;
 
@@ -255,11 +360,9 @@ private:
 
     ub4                 nStmtMode;
 
-    char*               pszStatement;
-
 public:
 
-    bool                Execute( int nRows = 0 );
+    bool                Execute( int nRows = 1 );
     bool                Fetch( int nRows = 1 );
     unsigned int        nFetchCount;
 
@@ -267,36 +370,47 @@ public:
     double              GetDouble( OCINumber* ppoData );
     char*               GetString( OCIString* ppoData );
 
-    void                Define( int* pnData );
     void                Bind( int* pnData );
+    void                Bind( long* pnData );
     void                Bind( double* pnData );
-    void                Bind( char* pData, long nData);    
+    void                Bind( char* pData, long nData );
+    void                Bind( sdo_geometry** pphData );
+    void                Bind( OCILobLocator** pphLocator );
+    void                Bind( OCIArray** pphData, OCIType* type );
+    void                Bind( char* pszData, int nSize = OWNAME );
+    void                Define( int* pnData );
+    void                Define( long* pnData );
     void                Define( double* pnData );
     void                Define( char* pszData, int nSize = OWNAME );
-    void                Bind( char* pszData, int nSize = OWNAME );
-    void                Define( OCILobLocator** pphLocator,
-                            bool bBLOB = false);
+    void                Define( OCILobLocator** pphLocator );
     void                Define( OCIArray** pphData );
     void                Define( sdo_georaster** pphData );
     void                Define( sdo_geometry** pphData );
-    void                Define( OCILobLocator** pphLocator,
-                            int nIterations );
-    void                BindName( char* pszName, int* pnData );
-    void                BindName( char* pszName, char* pszData,
+    void                Define( sdo_pc** pphData );
+    void                Define( OCILobLocator** pphLocator, long nIterations );
+    void                BindName( const char* pszName, int* pnData );
+    void                BindName( const char* pszName, double* pnData );
+    void                BindName( const char* pszName, char* pszData,
                             int nSize = OWNAME );
-    void                BindName( char* pszName,
+    void                BindName( const char* pszName,
                             OCILobLocator** pphLocator );
+    void                BindArray( void* pData, long nSize = 1);
     static void         Free( OCILobLocator** ppphLocator,
                             int nCount );
     unsigned long       ReadBlob( OCILobLocator* phLocator,
                             void* pBuffer, int nSize );
     char*               ReadCLob( OCILobLocator* phLocator );
+    void                WriteCLob( OCILobLocator** pphLocator, char* pszData );
     bool                WriteBlob( OCILobLocator* phLocator,
                             void* pBuffer, int nSize );
     int                 GetElement( OCIArray** ppoData,
                             int nIndex, int* pnResult );
     double              GetElement( OCIArray** ppoData,
                             int nIndex, double* pdfResult );
+    void                AddElement( OCIArray* ppoData,
+                            int nValue );
+    void                AddElement( OCIArray* ppoData,
+                            double dfValue );
 };
 
 #endif /* ifndef _ORCL_WRAP_H_INCLUDED */
