@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf4dataset.cpp 16233 2009-02-05 21:46:06Z rouault $
+ * $Id: hdf4dataset.cpp 22838 2011-07-30 17:55:35Z rouault $
  *
  * Project:  Hierarchical Data Format Release 4 (HDF4)
  * Purpose:  HDF4 Datasets. Open HDF4 file, fetch metadata and list of
@@ -39,7 +39,7 @@
 #include "hdf4compat.h"
 #include "hdf4dataset.h"
 
-CPL_CVSID("$Id: hdf4dataset.cpp 16233 2009-02-05 21:46:06Z rouault $");
+CPL_CVSID("$Id: hdf4dataset.cpp 22838 2011-07-30 17:55:35Z rouault $");
 
 CPL_C_START
 void	GDALRegister_HDF4(void);
@@ -63,6 +63,9 @@ HDF4Dataset::HDF4Dataset()
     fp = NULL;
     hSD = 0;
     hGR = 0;
+    nImages = 0;
+    iSubdatasetType = H4ST_UNKNOWN;
+    pszSubdatasetType = NULL;
     papszGlobalMetadata = NULL;
     papszSubDatasets = NULL;
     bIsHDFEOS = 0;
@@ -752,19 +755,28 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
     int		nCount;
     int32	aiDimSizes[H4_MAX_VAR_DIMS];
     int32	iRank, iNumType, nAttrs;
-
+    bool        bIsHDF = true;
+    
     // Sometimes "HDFEOSVersion" attribute is not defined and we will
     // determine HDF-EOS datasets using other records
     // (see ReadGlobalAttributes() method).
     if ( poDS->bIsHDFEOS
          || CSLFetchNameValue(poDS->papszGlobalMetadata, "HDFEOSVersion") )
     {
+        bIsHDF  = false;
+
         int32   nSubDatasets, nStrBufSize;
 
 /* -------------------------------------------------------------------- */
 /*  Process swath layers.                                               */
 /* -------------------------------------------------------------------- */
         hHDF4 = SWopen( poOpenInfo->pszFilename, DFACC_READ );
+        if( hHDF4 < 0)
+        {
+            delete poDS;
+            CPLError( CE_Failure, CPLE_OpenFailed, "Failed to open HDF4 `%s'.\n", poOpenInfo->pszFilename );
+            return NULL;
+        } 
         nSubDatasets = SWinqswath(poOpenInfo->pszFilename, NULL, &nStrBufSize);
 #if DEBUG
         CPLDebug( "HDF4", "Number of HDF-EOS swaths: %d", (int)nSubDatasets );
@@ -975,9 +987,11 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
             GDclose( hHDF4 );
         }
         GDclose( hHDF4 );
+
+        bIsHDF = ( nSubDatasets == 0 ); // Try to read as HDF
     }
 
-    else
+    if( bIsHDF )
     {
 
 /* -------------------------------------------------------------------- */
@@ -1081,12 +1095,36 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if ( CSLCount( poDS->papszSubDatasets ) / 2 == 1 )
     {
-	char *pszSDSName;
-	pszSDSName = CPLStrdup( CSLFetchNameValue( poDS->papszSubDatasets,
-				        "SUBDATASET_1_NAME" ));
-	delete poDS;
-	poDS = (HDF4Dataset *) GDALOpen( pszSDSName, GA_ReadOnly );
-	CPLFree( pszSDSName );
+        char *pszSDSName;
+        pszSDSName = CPLStrdup( CSLFetchNameValue( poDS->papszSubDatasets,
+                            "SUBDATASET_1_NAME" ));
+        delete poDS;
+        poDS = NULL;
+
+        GDALDataset* poRetDS = (GDALDataset*) GDALOpen( pszSDSName, poOpenInfo->eAccess );
+        CPLFree( pszSDSName );
+
+        if (poRetDS)
+        {
+            poRetDS->SetDescription(poOpenInfo->pszFilename);
+        }
+
+        return poRetDS;
+    }
+    else
+    {
+/* -------------------------------------------------------------------- */
+/*      Confirm the requested access is supported.                      */
+/* -------------------------------------------------------------------- */
+        if( poOpenInfo->eAccess == GA_Update )
+        {
+            delete poDS;
+            CPLError( CE_Failure, CPLE_NotSupported, 
+                      "The HDF4 driver does not support update access to existing"
+                      " datasets.\n" );
+            return NULL;
+        }
+    
     }
 
     return( poDS );
