@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf5imagedataset.cpp 22341 2011-05-09 16:54:11Z warmerdam $
+ * $Id: hdf5imagedataset.cpp 23068 2011-09-06 20:50:23Z antonio $
  *
  * Project:  Hierarchical Data Format Release 5 (HDF5)
  * Purpose:  Read subdatasets of HDF5 file.
@@ -37,10 +37,10 @@
 #include "hdf5dataset.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: hdf5imagedataset.cpp 22341 2011-05-09 16:54:11Z warmerdam $");
+CPL_CVSID("$Id: hdf5imagedataset.cpp 23068 2011-09-06 20:50:23Z antonio $");
 
 CPL_C_START
-void    GDALRegister_HDF5Image(void);
+void GDALRegister_HDF5Image(void);
 CPL_C_END
 
 /* release 1.6.3 or 1.6.4 changed the type of count in some api functions */
@@ -69,7 +69,7 @@ class HDF5ImageDataset : public HDF5Dataset
     hid_t        dataset_id;
     hid_t        dataspace_id;
     hsize_t      size;
-    int          address;
+    haddr_t      address;
     hid_t        datatype;
     hid_t        native;
     H5T_class_t  clas;
@@ -77,7 +77,7 @@ class HDF5ImageDataset : public HDF5Dataset
 public:
     HDF5ImageDataset();
     ~HDF5ImageDataset();
-    
+
     CPLErr CreateProjections( );
     static GDALDataset  *Open( GDALOpenInfo * );
     static int           Identify( GDALOpenInfo * );
@@ -85,8 +85,8 @@ public:
     const char          *GetProjectionRef();
     virtual int         GetGCPCount( );
     virtual const char  *GetGCPProjection();
-    virtual const GDAL_GCP *GetGCPs( ); 
-    
+    virtual const GDAL_GCP *GetGCPs( );
+
 };
 
 /************************************************************************/
@@ -122,10 +122,10 @@ HDF5ImageDataset::~HDF5ImageDataset( )
     CPLFree(pszGCPProjection);
 
     if( dims )
-	CPLFree( dims );
+        CPLFree( dims );
 
     if( maxdims )
-	CPLFree( maxdims );
+        CPLFree( maxdims );
 
     if( nGCPCount > 0 )
     {
@@ -135,7 +135,7 @@ HDF5ImageDataset::~HDF5ImageDataset( )
                 CPLFree( pasGCPList[i].pszId );
             if( pasGCPList[i].pszInfo )
                 CPLFree( pasGCPList[i].pszInfo );
-	}
+        }
 
         CPLFree( pasGCPList );
     }
@@ -155,13 +155,13 @@ class HDF5ImageRasterBand : public GDALPamRasterBand
     char        *pszFilename;
 
 public:
-  
+
     HDF5ImageRasterBand( HDF5ImageDataset *, int, GDALDataType );
     ~HDF5ImageRasterBand();
 
-    virtual CPLErr          IReadBlock( int, int, void * );
-    virtual double	    GetNoDataValue( int * ); 
-    virtual CPLErr	    SetNoDataValue( double );
+    virtual CPLErr      IReadBlock( int, int, void * );
+    virtual double      GetNoDataValue( int * );
+    virtual CPLErr      SetNoDataValue( double );
     /*  virtual CPLErr          IWriteBlock( int, int, void * ); */
 };
 
@@ -198,7 +198,7 @@ HDF5ImageRasterBand::HDF5ImageRasterBand( HDF5ImageDataset *poDS, int nBand,
     poDS->papszMetadata = NULL;
 
     if( poDS->poH5Objects->nType == H5G_DATASET ) {
-	poDS->CreateMetadata( poDS->poH5Objects, H5G_DATASET );
+        poDS->CreateMetadata( poDS->poH5Objects, H5G_DATASET );
     }
 
 /* -------------------------------------------------------------------- */
@@ -211,7 +211,7 @@ HDF5ImageRasterBand::HDF5ImageRasterBand( HDF5ImageDataset *poDS, int nBand,
     poDS->papszMetadata = CSLDuplicate( papszMetaGlobal );
     CSLDestroy( papszMetaGlobal );
 
-/* check for chunksize and set it as the blocksize (optimizes read) */
+    /* check for chunksize and set it as the blocksize (optimizes read) */
     hid_t listid = H5Dget_create_plist(((HDF5ImageDataset * )poDS)->dataset_id);
     if (listid>0)
     {
@@ -219,8 +219,8 @@ HDF5ImageRasterBand::HDF5ImageRasterBand( HDF5ImageDataset *poDS, int nBand,
         {
             hsize_t panChunkDims[3];
             int nDimSize = H5Pget_chunk(listid, 3, panChunkDims);
-            nBlockXSize   = panChunkDims[nDimSize-1];
-            nBlockYSize   = panChunkDims[nDimSize-2];
+            nBlockXSize   = (int) panChunkDims[nDimSize-1];
+            nBlockYSize   = (int) panChunkDims[nDimSize-2];
         }
         H5Pclose(listid);
     }
@@ -237,7 +237,7 @@ double HDF5ImageRasterBand::GetNoDataValue( int * pbSuccess )
     {
         if( pbSuccess )
             *pbSuccess = bNoDataSet;
-        
+
         return dfNoDataValue;
     }
     else
@@ -271,21 +271,21 @@ CPLErr HDF5ImageRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     hsize_t     rank;
 
     HDF5ImageDataset    *poGDS = ( HDF5ImageDataset * ) poDS;
-   
+
     if( poGDS->eAccess == GA_Update ) {
-	memset( pImage, 0,
-		nBlockXSize * nBlockYSize * 
-		GDALGetDataTypeSize( eDataType )/8 );
-	return CE_None;
+        memset( pImage, 0,
+            nBlockXSize * nBlockYSize *
+            GDALGetDataTypeSize( eDataType )/8 );
+        return CE_None;
     }
 
     rank=2;
 
     if( poGDS->ndims == 3 ){
-	rank=3;
-	offset[0]   = nBand-1;
-	count[0]    = 1;
-	col_dims[0] = 1;
+        rank=3;
+        offset[0]   = nBand-1;
+        count[0]    = 1;
+        col_dims[0] = 1;
     }
 
     offset[poGDS->ndims - 2] = nBlockYOff*nBlockYSize;
@@ -307,17 +307,17 @@ CPLErr HDF5ImageRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /* -------------------------------------------------------------------- */
 /*      Select block from file space                                    */
 /* -------------------------------------------------------------------- */
-    status =  H5Sselect_hyperslab( poGDS->dataspace_id, 
-				  H5S_SELECT_SET, 
-				  offset, NULL, 
-				  count, NULL );
-   
+    status =  H5Sselect_hyperslab( poGDS->dataspace_id,
+                                   H5S_SELECT_SET,
+                                   offset, NULL,
+                                   count, NULL );
+
 /* -------------------------------------------------------------------- */
 /*      Create memory space to receive the data                         */
 /* -------------------------------------------------------------------- */
     col_dims[poGDS->ndims-2]=nBlockYSize;
     col_dims[poGDS->ndims-1]=nBlockXSize;
-    memspace = H5Screate_simple( rank, col_dims, NULL );
+    memspace = H5Screate_simple( (int) rank, col_dims, NULL );
     H5OFFSET_TYPE mem_offset[3] = {0, 0, 0};
     status =  H5Sselect_hyperslab(memspace,
                                   H5S_SELECT_SET,
@@ -325,11 +325,13 @@ CPLErr HDF5ImageRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                   count, NULL);
 
     status = H5Dread ( poGDS->dataset_id,
-		      poGDS->native, 
-		      memspace,
-		      poGDS->dataspace_id,
-		      H5P_DEFAULT, 
-		      pImage );
+                       poGDS->native,
+                       memspace,
+                       poGDS->dataspace_id,
+                       H5P_DEFAULT,
+                       pImage );
+
+    H5Sclose( memspace );
 
     return CE_None;
 }
@@ -342,7 +344,7 @@ int HDF5ImageDataset::Identify( GDALOpenInfo *poOpenInfo )
 
 {
     if(!EQUALN( poOpenInfo->pszFilename, "HDF5:", 5 ) )
-	return FALSE;
+        return FALSE;
     else
         return TRUE;
 }
@@ -358,26 +360,26 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if(!EQUALN( poOpenInfo->pszFilename, "HDF5:", 5 ) ||
         strlen(poOpenInfo->pszFilename) > sizeof(szFilename) - 3 )
-	return NULL;
-    
+        return NULL;
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The HDF5ImageDataset driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     poDS = new HDF5ImageDataset();
 
     /* -------------------------------------------------------------------- */
     /*      Create a corresponding GDALDataset.                             */
     /* -------------------------------------------------------------------- */
     /* printf("poOpenInfo->pszFilename %s\n",poOpenInfo->pszFilename); */
-    char **papszName = 
+    char **papszName =
         CSLTokenizeString2(  poOpenInfo->pszFilename,
                              ":", CSLT_HONOURSTRINGS|CSLT_PRESERVEESCAPES );
 
@@ -389,17 +391,17 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo * poOpenInfo )
     }
 
     poDS->SetDescription( poOpenInfo->pszFilename );
-  
+
     /* -------------------------------------------------------------------- */
     /*    Check for drive name in windows HDF5:"D:\...                      */
     /* -------------------------------------------------------------------- */
     strcpy(szFilename, papszName[1]);
 
-    if( strlen(papszName[1]) == 1 && papszName[3] != NULL ) 
+    if( strlen(papszName[1]) == 1 && papszName[3] != NULL )
     {
-	strcat(szFilename, ":");
-	strcat(szFilename, papszName[2]);
-        
+        strcat(szFilename, ":");
+        strcat(szFilename, papszName[2]);
+
         poDS->SetSubdatasetName( papszName[3] );
     }
     else
@@ -419,19 +421,21 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo * poOpenInfo )
     /*      Try opening the dataset.                                        */
     /* -------------------------------------------------------------------- */
     poDS->hHDF5 = H5Fopen(szFilename,
-			  H5F_ACC_RDONLY, 
-			  H5P_DEFAULT );
-  
-    if( poDS->hHDF5 < 0 )  {
+                          H5F_ACC_RDONLY,
+                          H5P_DEFAULT );
+
+    if( poDS->hHDF5 < 0 )
+    {
         delete poDS;
-	return NULL;
+        return NULL;
     }
-  
-    poDS->hGroupID = H5Gopen( poDS->hHDF5, "/" ); 
-    if( poDS->hGroupID < 0 ){
-	poDS->bIsHDFEOS=false;
+
+    poDS->hGroupID = H5Gopen( poDS->hHDF5, "/" );
+    if( poDS->hGroupID < 0 )
+    {
+        poDS->bIsHDFEOS=false;
         delete poDS;
-	return NULL;
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -439,52 +443,51 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->bIsHDFEOS=TRUE;
     poDS->ReadGlobalAttributes( FALSE );
-  
+
 /* -------------------------------------------------------------------- */
 /*      Create HDF5 Data Hierarchy in a link list                       */
 /* -------------------------------------------------------------------- */
-    poDS->poH5Objects = 
-	poDS->HDF5FindDatasetObjectsbyPath( poDS->poH5RootGroup, 
+    poDS->poH5Objects =
+        poDS->HDF5FindDatasetObjectsbyPath( poDS->poH5RootGroup,
                                             (char *)poDS->GetSubdatasetName() );
 
     if( poDS->poH5Objects == NULL ) {
-	return NULL;
+        delete poDS;
+        return NULL;
     }
 /* -------------------------------------------------------------------- */
 /*      Retrieve HDF5 data information                                  */
 /* -------------------------------------------------------------------- */
-    poDS->dataset_id   = H5Dopen( poDS->hHDF5,poDS->poH5Objects->pszPath ); 
-    poDS->dataspace_id = H5Dget_space( poDS->dataset_id );                       
+    poDS->dataset_id   = H5Dopen( poDS->hHDF5,poDS->poH5Objects->pszPath );
+    poDS->dataspace_id = H5Dget_space( poDS->dataset_id );
     poDS->ndims        = H5Sget_simple_extent_ndims( poDS->dataspace_id );
-    poDS->dims         = ( hsize_t * )CPLCalloc( poDS->ndims, 
-						 sizeof( hsize_t ) );
-    poDS->maxdims      = ( hsize_t * )CPLCalloc( poDS->ndims, 
-						 sizeof( hsize_t ) );
+    poDS->dims         = (hsize_t*)CPLCalloc( poDS->ndims, sizeof(hsize_t) );
+    poDS->maxdims      = (hsize_t*)CPLCalloc( poDS->ndims, sizeof(hsize_t) );
     poDS->dimensions   = H5Sget_simple_extent_dims( poDS->dataspace_id,
-						   poDS->dims,
-						   poDS->maxdims );
+                                                    poDS->dims,
+                                                    poDS->maxdims );
     poDS->datatype = H5Dget_type( poDS->dataset_id );
     poDS->clas     = H5Tget_class( poDS->datatype );
     poDS->size     = H5Tget_size( poDS->datatype );
     poDS->address = H5Dget_offset( poDS->dataset_id );
     poDS->native  = H5Tget_native_type( poDS->datatype, H5T_DIR_ASCEND );
 
-    poDS->nRasterYSize=poDS->dims[poDS->ndims-2];   // Y
-    poDS->nRasterXSize=poDS->dims[poDS->ndims-1];   // X alway last
+    poDS->nRasterYSize=(int)poDS->dims[poDS->ndims-2];   // Y
+    poDS->nRasterXSize=(int)poDS->dims[poDS->ndims-1];   // X alway last
 
     poDS->nBands=1;
 
-    if( poDS->ndims == 3 ) poDS->nBands=poDS->dims[0];
+    if( poDS->ndims == 3 ) poDS->nBands=(int) poDS->dims[0];
 
 
     for(  i = 1; i <= poDS->nBands; i++ ) {
-	HDF5ImageRasterBand *poBand = 
-	    new HDF5ImageRasterBand( poDS, i, 
-				     poDS->GetDataType( poDS->native ) );
-	
-	poDS->SetBand( i, poBand );
-	if( poBand->bNoDataSet )
-		poBand->SetNoDataValue( 255 );
+        HDF5ImageRasterBand *poBand =
+            new HDF5ImageRasterBand( poDS, i,
+                            poDS->GetDataType( poDS->native ) );
+
+        poDS->SetBand( i, poBand );
+        if( poBand->bNoDataSet )
+            poBand->SetNoDataValue( 255 );
     }
 
     poDS->CreateProjections( );
@@ -512,18 +515,18 @@ void GDALRegister_HDF5Image( )
 
 {
     GDALDriver  *poDriver;
-    
+
     if (! GDAL_CHECK_VERSION("HDF5Image driver"))
         return;
 
     if(  GDALGetDriverByName( "HDF5Image" ) == NULL )
     {
         poDriver = new GDALDriver( );
-        
+
         poDriver->SetDescription( "HDF5Image" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
+        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                    "HDF5 Dataset" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
+        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
                                    "frmt_hdf5.html" );
         poDriver->pfnOpen = HDF5ImageDataset::Open;
         poDriver->pfnIdentify = HDF5ImageDataset::Identify;
@@ -561,82 +564,84 @@ CPLErr HDF5ImageDataset::CreateProjections()
 /* -------------------------------------------------------------------- */
     poH5Objects=HDF5FindDatasetObjects( poH5RootGroup,  "Latitude" );
     if( !poH5Objects ) {
-	return CE_None;
+        return CE_None;
     }
 /* -------------------------------------------------------------------- */
 /*      The Lattitude and Longitude arrays must have a rank of 2 to     */
 /*      retrieve GCPs.                                                  */
 /* -------------------------------------------------------------------- */
     if( poH5Objects->nRank != 2 ) {
-	return CE_None;
+        return CE_None;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Retrieve HDF5 data information                                  */
 /* -------------------------------------------------------------------- */
-    LatitudeDatasetID   = H5Dopen( hHDF5,poH5Objects->pszPath ); 
-    LatitudeDataspaceID = H5Dget_space( dataset_id );                       
+    LatitudeDatasetID   = H5Dopen( hHDF5,poH5Objects->pszPath );
+    LatitudeDataspaceID = H5Dget_space( dataset_id );
 
     poH5Objects=HDF5FindDatasetObjects( poH5RootGroup, "Longitude" );
-    LongitudeDatasetID   = H5Dopen( hHDF5,poH5Objects->pszPath ); 
-    LongitudeDataspaceID = H5Dget_space( dataset_id );                       
+    LongitudeDatasetID   = H5Dopen( hHDF5,poH5Objects->pszPath );
+    LongitudeDataspaceID = H5Dget_space( dataset_id );
 
     if( ( LatitudeDatasetID > 0 ) && ( LongitudeDatasetID > 0) ) {
-	
-	Latitude         = ( float * ) CPLCalloc(  nRasterYSize*nRasterXSize, 
-						sizeof( float ) );
-	Longitude         = ( float * ) CPLCalloc( nRasterYSize*nRasterXSize, 
-						 sizeof( float ) );
-	memset( Latitude, 0, nRasterXSize*nRasterYSize*sizeof(  float ) );
-	memset( Longitude, 0, nRasterXSize*nRasterYSize*sizeof( float ) );
-	
-	H5Dread ( LatitudeDatasetID,
-		  H5T_NATIVE_FLOAT, 
-		  H5S_ALL,
-		  H5S_ALL,
-		  H5P_DEFAULT, 
-		  Latitude );
-	
-	H5Dread ( LongitudeDatasetID,
-		  H5T_NATIVE_FLOAT, 
-		  H5S_ALL,
-		  H5S_ALL,
-		  H5P_DEFAULT, 
-		  Longitude );
-	
-	oSRS.SetWellKnownGeogCS( "WGS84" );
+
+        Latitude         = ( float * ) CPLCalloc(  nRasterYSize*nRasterXSize,
+                            sizeof( float ) );
+        Longitude         = ( float * ) CPLCalloc( nRasterYSize*nRasterXSize,
+                            sizeof( float ) );
+        memset( Latitude, 0, nRasterXSize*nRasterYSize*sizeof(  float ) );
+        memset( Longitude, 0, nRasterXSize*nRasterYSize*sizeof( float ) );
+
+        H5Dread ( LatitudeDatasetID,
+            H5T_NATIVE_FLOAT,
+            H5S_ALL,
+            H5S_ALL,
+            H5P_DEFAULT,
+            Latitude );
+
+        H5Dread ( LongitudeDatasetID,
+            H5T_NATIVE_FLOAT,
+            H5S_ALL,
+            H5S_ALL,
+            H5P_DEFAULT,
+            Longitude );
+
+        oSRS.SetWellKnownGeogCS( "WGS84" );
         CPLFree(pszProjection);
         CPLFree(pszGCPProjection);
-	oSRS.exportToWkt( &pszProjection );
-	oSRS.exportToWkt( &pszGCPProjection );
-	
-/* -------------------------------------------------------------------- */
-/*  Fill the GCPs list.                                                 */
-/* -------------------------------------------------------------------- */
-	nGCPCount = nRasterYSize/nDeltaLat * nRasterXSize/nDeltaLon;
+        oSRS.exportToWkt( &pszProjection );
+        oSRS.exportToWkt( &pszGCPProjection );
 
-	pasGCPList = ( GDAL_GCP * )
-	    CPLCalloc( nGCPCount, sizeof( GDAL_GCP ) );
-	
-	GDALInitGCPs( nGCPCount, pasGCPList );
-	int k=0;
+    /* -------------------------------------------------------------------- */
+    /*  Fill the GCPs list.                                                 */
+    /* -------------------------------------------------------------------- */
+        nGCPCount = nRasterYSize/nDeltaLat * nRasterXSize/nDeltaLon;
 
-	int nYLimit = ((int)nRasterYSize/nDeltaLat) * nDeltaLat;
-	int nXLimit = ((int)nRasterXSize/nDeltaLon) * nDeltaLon;
-	for( j = 0; j < nYLimit; j+=nDeltaLat ) {
-	    for( i = 0; i < nXLimit; i+=nDeltaLon ) {
-		int iGCP =  j * nRasterXSize + i;
-		pasGCPList[k].dfGCPX = ( double ) Longitude[iGCP]+180.0;
-		pasGCPList[k].dfGCPY = ( double ) Latitude[iGCP];
-		
-		pasGCPList[k].dfGCPPixel = i + 0.5;
-		pasGCPList[k++].dfGCPLine =  j + 0.5;
-		
-	    }
-	}
-	
-	CPLFree( Latitude );
-	CPLFree( Longitude );
+        pasGCPList = ( GDAL_GCP * )
+            CPLCalloc( nGCPCount, sizeof( GDAL_GCP ) );
+
+        GDALInitGCPs( nGCPCount, pasGCPList );
+        int k=0;
+
+        int nYLimit = ((int)nRasterYSize/nDeltaLat) * nDeltaLat;
+        int nXLimit = ((int)nRasterXSize/nDeltaLon) * nDeltaLon;
+        for( j = 0; j < nYLimit; j+=nDeltaLat )
+        {
+            for( i = 0; i < nXLimit; i+=nDeltaLon )
+            {
+                int iGCP =  j * nRasterXSize + i;
+                pasGCPList[k].dfGCPX = ( double ) Longitude[iGCP]+180.0;
+                pasGCPList[k].dfGCPY = ( double ) Latitude[iGCP];
+
+                pasGCPList[k].dfGCPPixel = i + 0.5;
+                pasGCPList[k++].dfGCPLine =  j + 0.5;
+
+            }
+        }
+
+        CPLFree( Latitude );
+        CPLFree( Longitude );
     }
     return CE_None;
 
@@ -646,7 +651,7 @@ CPLErr HDF5ImageDataset::CreateProjections()
 /************************************************************************/
 
 const char *HDF5ImageDataset::GetProjectionRef( )
-    
+
 {
     if( pszProjection )
         return pszProjection;
@@ -659,7 +664,7 @@ const char *HDF5ImageDataset::GetProjectionRef( )
 /************************************************************************/
 
 int HDF5ImageDataset::GetGCPCount( )
-    
+
 {
     if( nGCPCount > 0 )
         return nGCPCount;
@@ -675,9 +680,9 @@ const char *HDF5ImageDataset::GetGCPProjection( )
 
 {
     if( nGCPCount > 0 )
-	return pszGCPProjection;
+        return pszGCPProjection;
     else
-	return GDALPamDataset::GetGCPProjection();
+        return GDALPamDataset::GetGCPProjection();
 }
 
 /************************************************************************/
@@ -691,8 +696,3 @@ const GDAL_GCP *HDF5ImageDataset::GetGCPs( )
     else
         return GDALPamDataset::GetGCPs();
 }
-
-
-
-
-
