@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfa_p.h 19689 2010-05-13 12:07:48Z warmerdam $
+ * $Id: hfa_p.h 23495 2011-12-08 00:16:33Z rouault $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Private class declarations for the HFA classes used to read
@@ -35,6 +35,7 @@
 #include "cpl_port.h"
 #include "cpl_error.h"
 #include "cpl_vsi.h"
+#include <vector>
 
 #ifdef CPL_LSB
 #  define HFAStandard(n,p)	{}
@@ -63,7 +64,7 @@ typedef enum {
 /*      dataset within hfaopen.cpp                                      */
 /************************************************************************/
 typedef struct hfainfo {
-    FILE	*fp;
+    VSILFILE	*fp;
 
     char	*pszPath;
     char        *pszFilename; /* sans path */
@@ -154,7 +155,7 @@ class HFABand
                 
     HFAInfo_t	*psInfo;
 
-    FILE	*fpExternal;
+    VSILFILE	*fpExternal;
                          
     int		nDataType;
     HFAEntry	*poNode;
@@ -187,7 +188,7 @@ class HFABand
                         double ** );
     CPLErr	SetPCT( int, double *, double *, double *, double * );
 
-    int         CreateOverview( int nOverviewLevel );
+    int         CreateOverview( int nOverviewLevel, const char *pszResampling );
     CPLErr      CleanOverviews();
 
     CPLErr      LoadOverviews();
@@ -227,11 +228,19 @@ class HFAEntry
 
     void	LoadData();
 
-    int 	GetFieldValue( const char *, char, void * );
+    int 	GetFieldValue( const char *, char, void *,
+                               int *pnRemainingDataSize );
     CPLErr      SetFieldValue( const char *, char, void * );
 
     int         bIsMIFObject;
 
+                HFAEntry( HFAEntry *poContainer,
+                          const char *pszMIFObjectPath,
+                          const char * pszDictionnary, 
+                          const char * pszTypeName,
+                          int nDataSizeIn,
+                          GByte* pabyDataIn );
+
 public:
     		HFAEntry( HFAInfo_t * psHFA, GUInt32 nPos,
                           HFAEntry * poParent, HFAEntry *poPrev);
@@ -241,9 +250,10 @@ public:
                           const char *pszTypeName,
                           HFAEntry *poParent );
 
-                HFAEntry( HFAEntry *poContainer, const char *pszMIFObjectPath );
                           
     virtual     ~HFAEntry();                
+    
+    static HFAEntry*  BuildEntryFromMIFObject( HFAEntry *poContainer, const char *pszMIFObjectPath );
 
     CPLErr      RemoveAndDestroy();
 
@@ -262,10 +272,13 @@ public:
     HFAEntry	*GetChild();
     HFAEntry	*GetNext();
     HFAEntry    *GetNamedChild( const char * );
+    std::vector<HFAEntry*> FindChildren( const char *pszName, 
+                                         const char *pszType,
+                                         int nRecLevel = 0);
 
     GInt32	GetIntField( const char *, CPLErr * = NULL );
     double	GetDoubleField( const char *, CPLErr * = NULL );
-    const char	*GetStringField( const char *, CPLErr * = NULL );
+    const char	*GetStringField( const char *, CPLErr * = NULL, int *pnRemainingDataSize = NULL );
     GIntBig     GetBigIntField( const char *, CPLErr * = NULL );
     int         GetFieldCount( const char *, CPLErr * = NULL );
 
@@ -304,7 +317,7 @@ class HFAField
 
     char	*pszFieldName;
 
-    char        szNumberString[28]; /* buffer used to return an int as a string */
+    char        szNumberString[36]; /* buffer used to return an int as a string */
 
     		HFAField();
                 ~HFAField();
@@ -317,7 +330,7 @@ class HFAField
 
     int 	ExtractInstValue( const char * pszField, int nIndexValue,
                      GByte *pabyData, GUInt32 nDataOffset, int nDataSize,
-                     char chReqType, void *pReqReturn );
+                     char chReqType, void *pReqReturn, int *pnRemainingDataSize = NULL );
 
     CPLErr      SetInstValue( const char * pszField, int nIndexValue,
                      GByte *pabyData, GUInt32 nDataOffset, int nDataSize,
@@ -340,6 +353,8 @@ class HFAField
 
 class HFAType
 {
+    int     bInCompleteDefn;
+
   public:
     int		nBytes;
     
@@ -362,7 +377,7 @@ class HFAType
                           GByte *pabyData, GUInt32 nDataOffset, int nDataSize);
     int         ExtractInstValue( const char * pszField,
                                   GByte *pabyData, GUInt32 nDataOffset, int nDataSize,
-                               char chReqType, void *pReqReturn );
+                               char chReqType, void *pReqReturn, int *pnRemainingDataSize );
     CPLErr      SetInstValue( const char * pszField,
                            GByte *pabyData, GUInt32 nDataOffset, int nDataSize,
                            char chReqType, void * pValue );
