@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: l1bdataset.cpp 20996 2010-10-28 18:38:15Z rouault $
+ * $Id: l1bdataset.cpp 22715 2011-07-12 10:34:07Z rouault $
  *
  * Project:  NOAA Polar Orbiter Level 1b Dataset Reader (AVHRR)
  * Purpose:  Can read NOAA-9(F)-NOAA-17(M) AVHRR datasets
@@ -32,7 +32,7 @@
 #include "gdal_pam.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: l1bdataset.cpp 20996 2010-10-28 18:38:15Z rouault $");
+CPL_CVSID("$Id: l1bdataset.cpp 22715 2011-07-12 10:34:07Z rouault $");
 
 CPL_C_START
 void    GDALRegister_L1B(void);
@@ -646,8 +646,14 @@ void L1BDataset::ProcessRecordHeaders()
 /* -------------------------------------------------------------------- */
 /*      Initialize the GCP list.                                        */
 /* -------------------------------------------------------------------- */
-    pasGCPList = (GDAL_GCP *)CPLCalloc( nTargetLines * nGCPsPerLine,
+    pasGCPList = (GDAL_GCP *)VSICalloc( nTargetLines * nGCPsPerLine,
                                         sizeof(GDAL_GCP) );
+    if (pasGCPList == NULL)
+    {
+        CPLError( CE_Failure, CPLE_OutOfMemory, "Out of memory");
+        CPLFree( pRecordHeader );
+        return;
+    }
     GDALInitGCPs( nTargetLines * nGCPsPerLine, pasGCPList );
 
 /* -------------------------------------------------------------------- */
@@ -1584,9 +1590,6 @@ int L1BDataset::DetectFormat( GDALOpenInfo *poOpenInfo )
 int L1BDataset::Identify( GDALOpenInfo *poOpenInfo )
 
 {
-    if( poOpenInfo->fp == NULL )
-        return FALSE;
-
     if ( DetectFormat(poOpenInfo) == L1B_NONE )
         return FALSE;
 
@@ -1625,7 +1628,7 @@ GDALDataset *L1BDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
     L1BDataset  *poDS;
-    VSIStatBuf  sStat;
+    VSIStatBufL  sStat;
     const char  *pszFilename = poOpenInfo->pszFilename;
 
     poDS = new L1BDataset( eL1BFormat );
@@ -1646,7 +1649,7 @@ GDALDataset *L1BDataset::Open( GDALOpenInfo * poOpenInfo )
         goto bad;
     }
 
-    CPLStat(pszFilename, &sStat);
+    VSIStatL(pszFilename, &sStat);
 
     if ( poDS->bGuessDataFormat )
     {
@@ -1817,6 +1820,11 @@ GDALDataset *L1BDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->SetDescription( poOpenInfo->pszFilename );
     poDS->TryLoadXML();
 
+/* -------------------------------------------------------------------- */
+/*      Check for external overviews.                                   */
+/* -------------------------------------------------------------------- */
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename, poOpenInfo->papszSiblingFiles );
+
     return( poDS );
 
 bad:
@@ -1843,6 +1851,8 @@ void GDALRegister_L1B()
         poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
                                    "frmt_l1b.html" );
 
+        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
         poDriver->pfnOpen = L1BDataset::Open;
 
         GetGDALDriverManager()->RegisterDriver( poDriver );
